#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

# parser.py
# Python parser module for PyMol
#

import pymol
import traceback
import string
import cmd
import exceptions
import re
import parsing
import types
import glob
import sys
import os
import setting
import __main__

from cmd import _feedback,fb_module,fb_mask

QuietException = parsing.QuietException
      
pymol_names = pymol.__dict__

# parsing state implemented with dictionaries to enable safe recursion
# to arbitrary depths

com0 = {}    # verbose line, as read in
com1 = {}    # line w/o trailing whitespace
com2 = {}    # non-compound command
cont = {}    # continued characters from previous lines (i.e., before \ )
script = {}  # file handles
kw = {}      # row in the cmd.keyword table for the current command
input = {}   # list of length two - command and unparsed arguments string
next = {}    # characters for next command (i.e., after ; )
args = {}    # parsed non-keyword argument string
kw_args = {} # parser keyword argument string

# The resulting value from a pymol command (if any) is stored in the
# parser.result global variable.  However, script developers will
# geerally want to switch to the Python API for any of this kind of
# stuff.

result = None

# initialize parser

nest=0
com0[nest]=""
cont[nest]=""

# main parser routine

def parse(s):
   global com0,com1,com2,cont,script,kw,input
   global next,nest,args,kw_args,cmd,cont,result
# report any uncaught errors...
   if sys.exc_info()!=(None,None,None):
      traceback.print_exc()
#
   p_result = 1
   com0[nest] = s
   try:
      com1[nest] = string.rstrip(com0[nest]) # strips trailing whitespace
      if len(com1[nest]) > 0:
         if str(com1[nest][-1]) == "\\":
            cont[nest] = cont[nest] + com1[nest][:-1]
         else:
            if cont[nest] != '':
               com1[nest] = cont[nest] + com1[nest]
               cont[nest] = ''
# this routine splits up the line first based on semicolon 
            next[nest] = parsing.split(com1[nest],';',1)
# com2[nest] now a full non-compound command            
            com2[nest] = next[nest][0]
            input[nest] = string.split(com2[nest],' ',1)
            if len(input[nest]):
               input[nest][0] = string.strip(input[nest][0])
               com = input[nest][0]
               if com[0:1]=='/':
                  # explicit literal python 
                  com2[nest] = string.strip(com2[nest][1:])
                  if len(com2[nest])>0:
                      exec(com2[nest],pymol_names,pymol_names)
               else:
                  # try to find a keyword which matches
                  if cmd.kwhash.has_key(com):
                     amb = cmd.kwhash.interpret(com)
                     if amb == None:
                        com = cmd.kwhash[com]
                     elif type(amb)!=types.StringType:
                        print 'Error: ambiguous command: '
                        amb.sort()
                        amb = parsing.list_to_str_list(amb)
                        for a in amb:
                           print a
                        raise QuietException
                     com = amb
                  if cmd.keyword.has_key(com):
# here is the command and argument handling section
                     kw[nest] = cmd.keyword[com]
                     if kw[nest][4]>=parsing.NO_CHECK:
# stricter, Python-based argument parsing
                        if kw[nest][4]>=parsing.LITERAL: # treat literally
                           next[nest] = ()
                           com2[nest]=com1[nest]
                        (args[nest],kw_args[nest]) = \
                           parsing.prepare_call(
                              kw[nest][0],
                              parsing.parse_arg(com2[nest],mode=kw[nest][4]),
                              kw[nest][4]) # will raise exception on failure
                        result=apply(kw[nest][0],args[nest],kw_args[nest])
                     else:
# old parsing style, being phased out
                        if kw[nest][4]==parsing.ABORT:
                           return None # SCRIPT ABORT EXIT POINT
                        if kw[nest][4]==parsing.SINGLE: # single line, no breaks
                           next[nest] = ()
                           input[nest] = string.split(com1[nest],' ',1)
                        if len(input[nest])>1:
                           args[nest] = parsing.split(input[nest][1],kw[nest][3])
                           while 1:
                              nArg = len(args[nest]) - 1
                              c = 0
                              while c < nArg:
                                 if (string.count(args[nest][c],'(')!=
                                     string.count(args[nest][c],')')):
                                    tmp=args[nest][c+1]
                                    args[nest].remove(tmp)
                                    args[nest][c]=string.strip(args[nest][c])+\
                                                  ','+string.strip(tmp)
                                    nArg = nArg-1
                                    break;
                                 c = c + 1
                              if c == nArg:
                                 break;
                           if len(args[nest])==1 and len(args[nest][0])==0:
                              args[nest] = []
                        else:
                           args[nest] = []
                        if kw[nest][1]<= len(args[nest]) <= kw[nest][2]:
                           args[nest] = map(string.strip,args[nest])
                           if kw[nest][4]<parsing.RUN:
   #                           
   # this is where old-style commands are invoked
   #
                              result=apply(kw[nest][0],args[nest])
   #                           
                           elif kw[nest][4]==parsing.SPAWN:
                              # spawn command
                              if len(args[nest])==1: # default: module
                                 parsing.run_as_module(args[nest][0],spawn=1)
                              elif args[nest][1]=='main':
                                 parsing.run_as_thread(args[nest][0],
                                                       __main__.__dict__,
                                                       __main__.__dict__)
                              elif args[nest][1]=='private':
                                 parsing.run_as_thread(args[nest][0],
                                                       __main__.__dict__,
                                                       {})
                              elif args[nest][1]=='local':
                                 parsing.run_as_thread(args[nest][0],
                                                       pymol_names,{})
                              elif args[nest][1]=='global':
                                 parsing.run_as_thread(args[nest][0],
                                                       pymol_names,pymol_names)
                              elif args[nest][1]=='module':
                                 parsing.run_as_module(args[nest][0],spawn=1)
                           elif kw[nest][4]==parsing.RUN:
                              # run command
                              if len(args[nest])==1: # default: global
                                 execfile(args[nest][0],pymol_names,pymol_names)
                              elif args[nest][1]=='main':
                                 execfile(args[nest][0],__main__.__dict__,__main__.__dict__)
                              elif args[nest][1]=='private':
                                 execfile(args[nest][0],__main__.__dict__,{})
                              elif args[nest][1]=='local':
                                 execfile(args[nest][0],pymol_names,{})
                              elif args[nest][1]=='global':
                                 execfile(args[nest][0],pymol_names,pymol_names)
                              elif args[nest][1]=='module':
                                 parsing.run_as_module(args[nest][0],spawn=0)
                           else:
                              print 'Error: unknown keyword mode: '+str(kw[nest][4])
                              raise QuietException
                        else:
                           print 'Error: invalid arguments for %s command.' % com
#
# non-keyword command handling
#
                  elif len(input[nest][0]):
                     if input[nest][0][0]=='@':
                        script[nest] = open(string.strip(com2[nest][1:]),'r')
                        nest=nest+1
                        cont[nest]=''
                        while 1:
                           com0[nest]  = script[nest-1].readline()
                           if not com0[nest]: break
                           inp_cmd = com0[nest]
                           tmp_cmd = string.strip(inp_cmd)
                           if len(tmp_cmd):
                              if tmp_cmd[0] not in ['#','_','/']: # suppress comments, internals, python
                                 print "PyMOL>"+tmp_cmd
                              elif tmp_cmd[0]=='_' and \
                                   tmp_cmd[1:2] in [' ','']: # "_ " remove echo suppression signal
                                 inp_cmd=inp_cmd[2:]
                           pp_result = parse(inp_cmd)
                           if pp_result==None: # RECURSION
                              break # abort command gets us out
                           elif pp_result==0: # QuietException
                              if cmd.get_setting_legacy("stop_on_exceptions"):
                                 p_result = 0 # signal an error occurred
                                 print"PyMOL: stopped on exception."
                                 break;
                        nest=nest-1
                        script[nest].close()
                     else: # nothing found, try literal python
                        com2[nest] = string.strip(com2[nest])
                        if len(com2[nest])>0:
                           exec(com2[nest],pymol_names,pymol_names)
            if (len(next[nest])>1) and p_result:
                    # continue parsing if no error or break has occurred
               nest=nest+1
               com0[nest] = next[nest-1][1]
               next[nest-1]=()
               cont[nest]=''
               p_result = parse(com0[nest]) # RECURSION
               nest=nest-1
   except QuietException:
      if cmd._feedback(fb_module.parser,fb_mask.blather):
         print "PyMOL: Caught a quiet exception."
      p_result = 0 # notify caller that an error was encountered
   except:
      traceback.print_exc()
      if cmd._feedback(fb_module.parser,fb_mask.blather):
         print "PyMOL: Caught an unknown exception."
      p_result = 0 # notify caller that an error was encountered
   return p_result  # 0 = QuietException, None = abort, 1 = ok

# command and filename completion

def _same_(a,b):
   if a==b:
      return a
   else:
      return None

def complete_sc(st,sc,type_name,postfix):
   result = None
   sc=sc() # invoke lambda functions (if any)
   amb = sc.interpret(st)
   if amb==None:
      print " parser: no matching %s."%type_name
   elif type(amb)==types.StringType:
      result = amb+postfix
   else:
      amb.sort()
      print " parser: matching %s:"%type_name
      flist = filter(lambda x:x[0]!='_',amb)
      lst = parsing.list_to_str_list(flist)
      for a in lst:
         print a
      # now append up to point of ambiguity
      css = map(None,flist[0]) # common sub-string (css)
      for a in flist:
         ac = map(None,a)
         tmp = css
         css = []
         for c in range(len(tmp)):
            if tmp[c]!=ac[c]:
               break
            css.append(tmp[c])
      css = filter(None,css)
      css = string.join(css,'')
      if len(css)>len(st):
         result = css
   return result

def complete(st):
   result = None
   pre = ''
   flag = 0
   if (string.find(st,' ')<0) and (string.find(st,'@'))<0:
      try:
         result = complete_sc(st,cmd.kwhash,'commands',' ')
      except:
         traceback.print_exc()
   else:
      full = cmd.kwhash.interpret(re.sub(r" .*","",st))
      count = string.count(st,',') # which argument are we on
      if count<len(cmd.auto_arg):      
         if cmd.auto_arg[count].has_key(full): # autocomplete arguments
            flag = 1
            try:
               pre = re.sub(r"^[^ ]* ",' ',st,count=1) # trim command
               if re.search(r",",pre)!=None:
                  pre = re.sub(r"[^\, ]*$","",pre,count=1) 
                  pre = re.sub(r",\s*[^\, ]*$",", ",pre,count=1) # trim 1 arg
               else:
                  pre = re.sub("[^ ]*$","",pre,count=1) # trim 1 arg               
               pre = re.sub(r"^ *",'',pre)
               pre = full+' '+pre
               pat = re.sub(r".*[\, ]",'',st)
#               print ":"+pre+":"+pat+":"
               result = apply(complete_sc,tuple([pat]+cmd.auto_arg[count][full]),{})
            except:
               traceback.print_exc()
      if not flag: # otherwise fallback onto filename completion
         if(st[:1]=='@'):
            st=st[1:]
            pre = '@'
         loc = reduce(max,[string.rfind(st,','),
                           string.rfind(st,' '),
                           string.rfind(st,']'),
                           string.rfind(st,')')])+1
         st3 = st[loc:]
         flist = glob.glob(os.path.expanduser(os.path.expandvars(st3))+"*")
         lst = map(None,st3)
         lst.reverse()
         st3 = string.join(lst,'')
         lf = len(flist)
         if lf == 0:
            print " parser: no matching files."
         elif lf==1:
            result = st[0:loc]+flist[0]
            if os.path.isdir(flist[0]):
               result = result + "/"
         else:
            flist.sort()
            print " parser: matching files:"
            lst = parsing.list_to_str_list(flist)
            for a in lst:
               print a
            # now append as much up to point of ambiguity
            css = map(None,flist[0]) # common sub-string (css)
            for a in flist:
               ac = map(None,a)
               tmp = css
               css = []
               for c in range(len(tmp)):
                  if tmp[c]!=ac[c]:
                     break
                  css.append(tmp[c])
            tmp = css
            css = []
            for a in tmp:
               if a==None:
                  break
               css.append(a)
            css = string.join(css,'')
            if len(css)>len(st3):
               result = st[0:loc]+css
   if result!=None:
      result = pre+result
   return result



from chempy.models import Indexed,Connected
from chempy import Storage,Atom,Bond

import string
import copy

class MMD(Storage):
   
#---------------------------------------------------------------------------------
   def fromList(self,MMODList):

      model = Connected()
      
# get header information
      nAtom = int(MMODList[0][1:6])
      model.molecule.title = string.strip(MMODList[0][8:])
      irec = 1

# loop through atoms
      cnt = 0
      for a in range(nAtom):
         model.bond.append([])
         
      for a in range(nAtom):
         at = Atom()
         at.numeric_type = int(MMODList[irec][1:4])

# extract connectivity information
         tokens = string.splitfields(MMODList[irec][5:52])
         at.neighbor = []
         at.bondorder = []

         for i in range(6):
            if tokens[2*i] != "0":
               a2 = int(tokens[2*i])-1
               if (a2>cnt):
                  b = Bond()
                  b.index = [cnt,a2]
                  b.order = int(tokens[2*i+1])
                  model.bond[b.index[0]].append(b) # note two refs to same object
                  model.bond[b.index[1]].append(b) # note two refs to same object 
            else:
               break
         
# extract other information
         at.coord = [float(MMODList[irec][53:64]), 
            float(MMODList[irec][65:76]), float(MMODList[irec][77:88])]
         at.resi = string.strip(MMODList[irec][89:94])
         at.resi_number = int(at.resi)
         resn_code = string.strip(MMODList[irec][94:95])
         if len(resn_code): at.resn_code = resn_code
         color_code = string.strip(MMODList[irec][96:100])
         if color_code!='':
            at.color_code = int(color_code)
         else:
            at.color_code = 0
         chain = string.strip(MMODList[irec][95:96])
         if len(chain): at.chain = chain
         at.partial_charge = float(MMODList[irec][100:109])
         at.resn = MMODList[irec][119:123]
         name = string.strip(MMODList[irec][124:128])
         if len(name): at.name = name
         model.atom.append(at)
         irec = irec + 1
         cnt = cnt + 1
         
# fill in remaining datatypes
      cnt = 1
      for a in model.atom:
         a.text_type = MMOD_atom_data[a.numeric_type][0]
         a.symbol = MMOD_atom_data[a.numeric_type][1]
         a.formal_charge = MMOD_atom_data[a.numeric_type][4]
         cnt = cnt + 1
         
      return(model.convert_to_indexed())

#---------------------------------------------------------------------------------
   def updateFromList(self,model,list): # updates charges and coordinates
      nAtom = int(list[0][1:6])
      try:
         model.molecule.energy = float(list[0][58:68])/4.184 # convert to kcal
      except:
         if hasattr(model.molecule,'energy'):
            del model.molecule.energy
      if nAtom!=model.nAtom:
         raise RuntimeError(" mmd: atom count mismatch")
      c = 0
      for a in list[1:]:
         mac = model.atom[c]
         mac.coord = [float(a[53:64]), 
            float(a[65:76]), float(a[77:88])]
         mac.partial_charge = float(a[100:109])         
         c = c + 1
         
#---------------------------------------------------------------------------------
   def toList(self,model,no_blank_names=1):

      conn = copy.deepcopy(model)
      conn = conn.convert_to_connected()
      
      MMODList = []
      MMODList.append(" %5d  %-70s\n" %(conn.nAtom,conn.molecule.title))
      c = 0
      for i in conn.atom:
         
# construct neighbor list
         neighbors = 6*[0]
         bondorders = 6*[0]
         j = 0
         for b in conn.bond[c]:
            n = b.index[0]
            if n == c:
               n = b.index[1]
            neighbors[j] = n + 1
            bondorders[j] = b.order
            j = j + 1
            
# assemble output line
         if i.numeric_type>0:
            tline = " %3d" % (i.numeric_type)
         else:
            tline = " %3d" % 64
         for j in range(6): 
            tline = tline + " %5d %1d" % (neighbors[j], bondorders[j])
         tline = tline + " %11.6f %11.6f %11.6f " % (i.coord[0], 
            i.coord[1], i.coord[2])
         name = i.name
         if not len(name):
            if no_blank_names:
               name = "%s%d" % (i.symbol,c+1)
               name = name[0:4]
         tline = tline + "%5d%1s%1s%4d%9.5f%9.5f %-4s %4s\n" % \
            (i.resi_number,i.resn_code, i.chain, i.color_code,
             i.partial_charge, i.partial_charge, i.resn, name)
         MMODList.append(tline)
         c = c + 1
         
      return(MMODList)


#---------------------------------------------------------------------------------
'#Ntype Atype Elem Hybr Att Chg\n',
MMOD_atom_data = {
   1: ['C1','C' ,'sp' , 2, 0],
   2: ['C2','C' ,'sp2', 3, 0],
   3: ['C3','C' ,'sp3', 4, 0],
   4: ['CA','C' ,'sp3', 3, 0],
   5: ['CB','C' ,'sp3', 2, 0],
   6: ['CC','C' ,'sp3', 1, 0],
   7: ['CD','C' ,'sp2', 2, 0],
   8: ['CE','C' ,'sp2', 1, 0],
   9: ['CF','C' ,'sp' , 1, 0],
  10: ['CM','C' ,'unk',-1,-1],
  11: ['CP','C' ,'unk',-1, 1],
  12: ['CR','C' ,'unk',-1, 0],
  14: ['C0','C' ,'unk',-1, 0],
  15: ['O2','O' ,'sp2', 1, 0],
  16: ['O3','O' ,'sp3', 2, 0],
  17: ['OA','O' ,'sp3', 1, 0],
  18: ['OM','O' ,'sp3', 1,-1],
  19: ['OW','O' ,'sp3', 0, 0],
  20: ['OP','O' ,'sp2', 2, 1],
  21: ['OQ','O' ,'sp3', 3, 1],
  23: ['O0','O' ,'unk',-1, 0],
  24: ['N1','N' ,'sp' , 1, 0],
  25: ['N2','N' ,'sp2', 2, 0],
  26: ['N3','N' ,'sp3', 3, 0],
  27: ['NA','N' ,'sp3', 2, 0],
  28: ['NB','N' ,'sp3', 1, 0],
  29: ['NC','N' ,'sp3', 0, 0],
  30: ['ND','N' ,'sp2', 1, 0],
  31: ['N4','N' ,'sp2', 3, 1],
  32: ['N5','N' ,'sp3', 4, 1],
  33: ['NE','N' ,'sp3', 3, 1],
  34: ['NF','N' ,'sp3', 2, 1],
  35: ['NG','N' ,'sp3', 1, 1],
  36: ['NH','N' ,'sp2', 2, 1],
  37: ['NI','N' ,'sp2', 1, 1],
  40: ['N0','N' ,'unk',-1, 0],
  41: ['H1','H' ,'s'  , 1, 0],
  42: ['H2','H' ,'s'  , 1, 0],
  43: ['H3','H' ,'s'  , 1, 0],
  44: ['H4','H' ,'s'  , 0, 0],
  45: ['H5','H' ,'s'  , 0, 0],
  48: ['H0','H' ,'s'  ,-1, 0],
  49: ['S1','S' ,'sp3', 2, 0],
  50: ['SA','S' ,'sp3', 1, 0],
  51: ['SM','S' ,'sp3', 0,-1],
  52: ['S0','S' ,'unk',-1, 0],
  53: ['P0','P' ,'unk',-1, 0],
  54: ['B2','B' ,'sp2', 2, 0],
  55: ['B3','B' ,'sp3', 3, 0],
  56: ['F0','F' ,'sp3', 1, 0],
  57: ['Cl','Cl','sp3', 1, 0],
  58: ['Br','Br','sp3', 1, 0],
  59: ['I0','I' ,'sp3', 1, 0],
  60: ['Si','Si','unk',-1, 0],
  61: ['Du','Du','unk',-1, 0],
  62: ['Du','Du','unk',-1, 0],
  63: ['Lp','Lp','unk', 1, 0],
  64: ['Du','Du','unk',-1, 0]};





from pymol.wizard import Wizard
from pymol import cmd
import pymol

default_mode = 'labchg'

class Charge(Wizard):

   def __init__(self):

      Wizard.__init__(self)
      
      self.modes = [
         'labchg',
         'addchg',
         'cpychg',
         'zrochg',
         'mzochg',
         'cbachg',
         'movchg',
         'rbachg',
         'sumchg',
         ]

      self.mode = default_mode
      self.status = 0
      
      self.mode_name = {
         'labchg':'Show',
         'cpychg':'Copy',
         'addchg':'Add',
         'zrochg':'Zero',
         'mzochg':'Move & Zero (atom)',
         'cbachg':'Move & Zero (resi)',
         'movchg':'Move & Remove (atom)',
         'rbachg':'Move & Remove (resi)',
         'sumchg':'Get Total Charge'
         }
      
      # initialize mode menu
      
      smm = []
      smm.append([ 2, 'Atom Charge Mode', '' ])
      for a in self.modes:
         smm.append([ 1, self.mode_name[a], 'cmd.get_wizard().set_mode("'+a+'")'])

      self.menu['mode']=smm
      
      self.memory = 0
         

   def get_panel(self):
      return [
         [ 1, 'Charge Wizard',''],
         [ 3, self.mode_name[self.mode],'mode'],
         [ 2, 'Clear','cmd.get_wizard().clear()'],
         [ 2, 'Done','cmd.set_wizard()'],
         ]

   def cleanup(self):
      global default_mode
      default_mode = self.mode
      self.clear()
      
   def clear(self):
      self.set_status(0)
      if 'wcharge' in cmd.get_names('selections'):
         if self.mode!='sumchg':
            cmd.edit("wcharge")
            cmd.label("pkchain",'') # fastest clear command
         else:
            cmd.label("wcharge",'') # fastest clear command            
         cmd.delete("wcharge")
         cmd.unpick()
      cmd.unpick()
      cmd.refresh_wizard()
      
   def get_prompt(self):
      self.prompt = None
      if self.mode == 'cpychg':
         if self.status==0:
            self.prompt = [ 'Pick source atom...' ]
         elif self.status==1:
            self.prompt = [ 'Pick destination atom on which to assign charge %6.4f'%self.partial_charge ]
      if self.mode == 'addchg':
         if self.status==0:
            self.prompt = [ 'Pick source atom...' ]
         elif self.status==1:
            self.prompt = [ 'Pick destination atom on which to add charge %6.4f'%self.partial_charge ]
      if self.mode == 'mzochg':
         if self.status==0:
            self.prompt = [ 'Pick source atom to copy and zero...' ]
         elif self.status==1:
            self.prompt = [ 'Pick destination atom on which to add charge %6.4f'%self.partial_charge ]
      if self.mode == 'movchg':
         if self.status==0:
            self.prompt = [ 'Pick source atom to copy and destroy...' ]
         elif self.status==1:
            self.prompt = [ 'Pick destination atom on which to add charge %6.4f'%self.partial_charge ]
      if self.mode == 'sumchg':
         if self.status==0:
            self.prompt = [ 'Pick an atom on the chain...' ]
         if self.status==1:
            self.prompt = [ 'Total charge on the chain is %6.4f'%self.partial_charge,
                            'Pick an atom on the chain...' ]
      if self.mode == 'cbachg':
         if self.status==0:
            self.prompt = [ 'Pick source residue to copy and zero...' ]
         elif self.status==1:
            self.prompt = [ 'Pick destination residue on which to add charges.']
      if self.mode == 'rbachg':
         if self.status==0:
            self.prompt = [ 'Pick source residue to copy and remove...' ]
         elif self.status==1:
            self.prompt = [ 'Pick destination residue on which to add charges.']
      if self.mode == 'zrochg':
            self.prompt = [ 'Pick atom on which to zero charge...' ]


      if self.mode == 'labchg':
            self.prompt = [ 'Pick atom on which to show charge...' ]

      if "wcharge" in cmd.get_names('selections'):
         pymol.stored.charge = 0
         if cmd.iterate("(byres wcharge)",
                        "stored.charge = stored.charge + partial_charge"):
            self.prompt.insert(0,"Total charge on the residue is %6.4f"%pymol.stored.charge)
            
      return self.prompt
   
   def set_mode(self,mode):
      if mode in self.modes:
         self.mode = mode
      self.status = 0
      cmd.refresh_wizard()
      
   def set_status(self,status):
      self.status = status
      cmd.refresh_wizard()

   def do_pick(self,bondFlag):
      if bondFlag:
         print " Error: please select a single atom"
         
      if self.mode == 'cpychg':
         # picking up
         if self.status==0:
            if cmd.iterate("(pk1)","stored.charge = partial_charge"):
               self.partial_charge = pymol.stored.charge
               self.status = 1
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")
               
         # dropping off
         elif self.status==1:
            pymol.stored.charge=self.partial_charge
            if cmd.alter("(pk1)","partial_charge = stored.charge"):
               self.status = 0
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")

      if self.mode == 'cbachg' or self.mode == 'rbachg':
         # picking up
         if self.status==0:
            pymol.stored.chg_dict = {}
            if cmd.iterate("(byres pk1)","stored.chg_dict[name] = partial_charge"):
               self.charge_dict = pymol.stored.chg_dict
               self.status = 1
               cmd.label("(byres pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(byres pk1)")
               cmd.unpick()
               cmd.enable("wcharge")
               
         # dropping off
         elif self.status==1:
            pymol.stored.valid_atoms = []
            if cmd.iterate("(byres pk1)","stored.valid_atoms.append(name)"):
               kees = self.charge_dict.keys()
               valid_dict = {}
               for a in pymol.stored.valid_atoms:
                  if a in kees:
                     valid_dict[a] = 1
               pymol.stored.chg_dict = self.charge_dict
               # copy/add charges
               for a in valid_dict.keys():
                  cmd.alter("((byres pk1) and name %s)"%a,
                            "partial_charge = partial_charge + stored.chg_dict[name]")
                  if self.mode == 'rbachg':
                     cmd.remove("((wcharge) and name %s)"%a)
                  else:
                     cmd.alter("((wcharge) and name %s)"%a,
                               "partial_charge = 0")
               self.status = 0
               # update labels
               cmd.label("(wcharge or (byres pk1))","'%6.4f'%partial_charge")
               # show which atoms had charges moved
               cmd.select("wcharge","(none)")
               for a in valid_dict.keys():
                  cmd.select("wcharge","(wcharge or ((byres pk1) and name %s))"%a)
               cmd.unpick()
               cmd.enable("wcharge")

      if self.mode == 'addchg':
         # picking up
         if self.status==0:
            if cmd.iterate("(pk1)","stored.charge = partial_charge"):
               self.partial_charge = pymol.stored.charge
               self.status = 1
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")
               
         # dropping off
         elif self.status==1:
            pymol.stored.charge=self.partial_charge
            if cmd.alter("(pk1)","partial_charge = partial_charge + stored.charge"):
               self.status = 0
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")
               
      if self.mode == 'mzochg':
         # picking up
         if self.status==0:
            if cmd.iterate("(pk1)","stored.charge = partial_charge"):
               self.partial_charge = pymol.stored.charge
               self.status = 1
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")
               
         # dropping off
         elif self.status==1:
            pymol.stored.charge=self.partial_charge
            if cmd.alter("(pk1)","partial_charge = partial_charge + stored.charge"):
               self.status = 0
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.alter("(wcharge)","partial_charge=0")
               cmd.label("(wcharge)","'%6.4f'%partial_charge")               
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")

      if self.mode == 'movchg':
         # picking up
         if self.status==0:
            if cmd.iterate("(pk1)","stored.charge = partial_charge"):
               self.partial_charge = pymol.stored.charge
               self.status = 1
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")
               
         # dropping off
         elif self.status==1:
            pymol.stored.charge=self.partial_charge
            if cmd.alter("(pk1)","partial_charge = partial_charge + stored.charge"):
               self.status = 0
               cmd.label("(pk1)","'%6.4f'%partial_charge")
               cmd.remove("wcharge")
               cmd.select("wcharge","(pk1)")
               cmd.unpick()
               cmd.enable("wcharge")

      if self.mode == 'zrochg':
         if cmd.alter("(pk1)","partial_charge = 0.0"):
            cmd.label("(pk1)","'%6.4f'%partial_charge")
            cmd.select("wcharge","(pk1)")
            cmd.unpick()
            cmd.enable("wcharge")
               
      if self.mode == 'labchg':
         cmd.label("(pk1)","'%6.4f'%partial_charge")
         cmd.select("wcharge","(pk1)")
         cmd.unpick()
         cmd.enable("wcharge")
               
      if self.mode == 'sumchg':
         pymol.stored.charge = 0.0
         if cmd.iterate("(pkchain)","stored.charge = stored.charge + partial_charge"):
            self.partial_charge = pymol.stored.charge
            self.status = 1
            cmd.select("wcharge","(pkchain)")
            cmd.unpick()
            cmd.enable("wcharge")
         
      cmd.refresh_wizard()



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import string
import thread
import cmd

def show_help(cmmd): # INTERNAL
   print "PyMOL>help %s" % cmmd
   help(cmmd)
   if cmd.get_setting_legacy("internal_feedback")>0.1:
      print "(Hit ESC to hide)"

def help(command = "commands"):
   '''
DESCRIPTION

   "help" prints out the online help for a given command.
   
USAGE
 
   help command
   '''
   if cmd.get_setting_legacy("internal_feedback")>0.1:
      cmd.set("text","1",quiet=1)
   cmmd = cmd.help_sc.auto_err(command,'topic')   
   if cmd.keyword.has_key(cmmd):
      doc = cmd.keyword[cmmd][0].__doc__
      if doc:
         print "\n",string.strip(doc),"\n"
      else:
         print "Error: sorry no help available on that command."
   elif cmd.help_only.has_key(cmmd):
      doc = cmd.help_only[cmmd][0].__doc__
      if doc:
         print "\n",string.strip(doc),"\n"
      else:
         print "Error: sorry no help available on that command."      
   else:
      print "Error: unrecognized command"


def commands():
   '''
COMMANDS
 
   INPUT/OUTPUT  load      save      delete    quit
   VIEW          turn      move      clip      rock
                 show      hide      enable    disable
                 reset     refresh   rebuild   
                 zoom      origin    orient   
                 view      get_view  set_view
   MOVIES        mplay     mstop     mset      mdo
                 mpng      mmatrix   frame
                 rewind    middle    ending
                 forward   backward
   IMAGING       png       mpng
   RAY TRACING   ray       
   MAPS          isomesh   isodot
   DISPLAY       cls       viewport  splash    
   SELECTIONS    select    mask   
   SETTINGS      set       button
   ATOMS         alter     alter_state 
   EDITING       create    replace   remove    h_fill   remove_picked
                 edit      bond      unbond    h_add    fuse       
                 undo      redo      protect   cycle_valence  attach
   FITTING       fit       rms       rms_cur   pair_fit  
                 intra_fit intra_rms intra_rms_cur   
   COLORS        color     set_color
   HELP          help      commands
   DISTANCES     dist      
   STEREO        stereo
   SYMMETRY      symexp
   SCRIPTS       @         run
   LANGUAGE      alias     extend

Try "help <command-name>".  Also see the following extra topics:
 
   "movies", "keyboard", "mouse", "selections",
   "examples", "launching", "editing", and "api".
'''
   cmd.help('commands')

def editing():
   '''
SUMMARY

PyMOL has a rudimentary, but quite functional molecular structure
editing capability.  However, you will need to use an external mimizer
to "clean-up" your structures after editing.  Furthermore, if you are
going to modify molecules other than proteins, then you will also need
a way of assigning atom types on the fly.

To edit a conformation or structure, you first need to enter editing
mode (see Mouse Menu).  Then you can pick an atom (CTRL-Middle click)
or a bond (CTRL-Right click).  Next, you can use the other
CTRL-key/click combinations listed on the right hand side of the
screen to adjust the attached fragments.  For example, CTRL-left click
will move fragments about the selected torsion.

Editing structures is done through a series of CTRL key actions
applied to the currently selected atom or bonds. See "help edit_keys"
for the exact combinations.  To build structures, you usually just
replace hydrogens with methyl groups, etc., and then repeat.  They are
no short-cuts currently available for building common groups, but that
is planned for later versions.

NOTE
  
Only "lines" and "sticks" representations can be picked using the
mouse, however other representations will not interfere with picking
so long as one of these representation is present underneath.
   
'''
   cmd.help('editing')
   
def release():
   '''
RELEASE NOTES

PyMOL is a free, open, and expandable molecular graphics system
written by a computational scientist to enable molecular modeling from
directly within Python.  It will be of most benefit to hybrid
scientist/developers in the fields of structural biology,
computational chemistry, and informatics who seek an open and
unrestricted visualization tool for interfacing with their own
programs.  PyMOL will also be of benefit to advanced non-developers
familiar with similar programs such as Midas, O, Grasp, X-PLOR and
CNS.

Due to PyMOL's current "user-unfriendliness", this release is most
appropriate for those who prefer to use text commands and scripts, and
for developers who want to integrate PyMOL's visualization and
molecular editing capabilities with their own work.

PyMOL currently includes a diverse command language, a powerful
application programmers interface (API), and a variety of mouse and
keyboard driven functionality for viewing, animation, rendering, and
molecular editing.  A partial manual is now available on the web.

Two external GUI development options are supported for PyMOL:
"Tkinter" and "wxPython".  Developers can take their pick.  I am
committed to insuring that PyMOL will work with both of them, but it
is unlikely that I will have time to develop a complete external GUI
myself any time soon using either toolkit.

Note that only Tkinter is supported under Windows with the default
PyMOL and Python distributions, so for maximum ease of installation
under Windows, stick with Tkinter (Tcl/Tk).  For this reason, the
Tkinter-based GUI is going to be the default GUI for standard PyMOL
despite its drawbacks.

Warren L. DeLano (5/1/2001), warren@delanoscientific.com
'''
   cmd.help('release')

def edit_keys():
   '''
EDITING KEYS 

   These are defaults, which can be redefined.  Note that while
entering text on the command line, some of these control keys take on
text editing functions instead (CTRL - A, E, and K, and DELETE), so
you should clear the command line before trying to edit atoms.

ATOM REPLACEMENT
 
   CTRL-C    Replace picked atom with carbon   (C)
   CTRL-N    Replace picked atom with nitrogen (N)
   CTRL-O    Replace picked atom with oxygen   (O)
   CTRL-S    Replace picked atom with sulpher  (S)
   CTRL-G    Replace picked atom with hydrogen (H)
   CTRL-F    Replace picked atom with fluorene (F)
   CTRL-L    Replace picked atom with chlorine (Cl)
   CTRL-B    Replace picked atom with bromine  (Br)
   CTRL-I    Replace picked atom with iodine   (I)
   
ATOM MODIFICATION
  
   CTRL-J    Set charge on picked atom to -1
   CTRL-K    Set charge on picked atom to +1
   CTRL-D    Remove atom or bond (DELETE works too).
   CTRL-Y    Add a hydrogen to the current atom
   CTRL-R    Adjust hydrogens on atom/bond to match valence.
   CTRL-E    Inverts the picked stereo center, but you must first
             indicate the constant portions with the (lb) and (rb)
             selections.

   CTRL-T    Connect atoms in the (lb) and (rb) selections.
   CTRL-W    Cycle the bond valence on the picked bond.

UNDO and REDO of conformational changes (not atom changes!)

   CTRL-Z    undo the previous conformational change.
             (you can not currently undo atom modifications).
   CTRL-A    redo the previous conformational change.

'''
   cmd.help('edit_keys')


def at_sign():
   '''
DESCRIPTION
 
   "@" sources a PyMOL command script as if all of the commands in the
   file were typed into the PyMOL command line.

USAGE
  
   @ <script-file>

PYMOL API

   Not directly available. Instead, use cmd.do("@...").
 
'''
   cmd.help(at_sign)


def run():
   '''
DESCRIPTION
 
   "run" executes an external Python script in a local name space, the
   global namespace, or in its own namespace (as a module).

USAGE
  
   run python-script [, (local | global | module | main | private ) ]

PYMOL API

   Not directly available.  Instead, use cmd.do("run ...").

NOTES

   The default mode for run is "global".
 
   Due to an idiosyncracy in Pickle, you can not pickle objects
   directly created at the main level in a script run as "module",
   (because the pickled object becomes dependent on that module).
   Workaround: delegate construction to an imported module.

'''
   cmd.help(run)

def spawn():
   '''
DESCRIPTION
 
   "spawn" launches a Python script in a new thread which will run
   concurrently with the PyMOL interpreter. It can be run in its own
   namespace (like a Python module, default), a local name space, or
   in the global namespace.
 
USAGE
  
   run python-script [, ( local | global | module | main | private )]

PYMOL API

   Not directly available.  Instead, use cmd.do("spawn ...").

NOTES

   The default mode for spawn is "module".
 
   Due to an idiosyncracy in Pickle, you can not pickle objects
   directly created at the main level in a script run as "module",
   (because the pickled object becomes dependent on that module).
   Workaround: delegate construction to an imported module.

   The best way to spawn processes at startup is to use the -l option
   (see "help launching").
'''
   cmd.help(spawn)

def api():
   '''
DESCRIPTION
 
   The PyMOL Python Application Programming Interface (API) should be
   accessed exclusively through the "cmd" module (never "_cmd"!).  Nearly
   all command-line functions have a corresponding API method.
 
USAGE
 
   from pymol import cmd
   result = cmd.<command-name>( argument , ... ) 
    
NOTES
 
   Although the PyMOL core is not multi-threaded, the API is
   thread-safe and can be called asynchronously by external python
   programs.  PyMOL handles the necessary locking to insure that
   internal states do not get corrupted.  This makes it very easy to
   build complicated systems which involve direct realtime visualization.

   '''
   
   cmd.help('api')

def keyboard():
   '''
KEYBOARD COMMANDS and MODIFIERS
 
   ESC          Toggle onscreen text.
   INSERT       Toggle rocking.

   LEFT ARROW, RIGHT ARROW    Go backward or forward one frame, or when
                              editing, go forward or back one character.
   HOME, END    Go to the beginning or end of a movie.

   Command Entry Field in the Interal GUI (black window)
   
   TAB          Complete commmand or filename (like in tcsh or bash).
   CTRL-A       Go to the beginning of the line.
   CTRL-E       Go to the end of the line.
   CTRL-K       Delete through to the end of the line.

   Command Entry Field on the External GUI (gray window).
   
   CTRL-C       These operating system-provided cut and paste functions
   CTRL-V       will only work in the external GUI command line.
   
EDITING 

   type "help edit_keys" for keyboard shortcuts used in editing.
   
   '''
   cmd.help('keyboard')

def transparency():
   '''
TRANSPARENCY
 
   As of version 0.68, trasparent surfaces are supported in both
   realtime (OpenGL) rendering mode as well as with ray-traced images.

   Transparency is currently managed by setting either the global
   transparency variable or one attached to an individual molecule object.

   It isn't yet possible to control transparency on a per-atom basis.
   
EXAMPLES

   set transparency=0.5        # makes all surfaces 50% transparent
   set transparency=0.5, mol3  # makes only mol3's surface transparent

   '''
   cmd.help('transparency')


def mouse():
   '''
MOUSE CONTROLS
 
  The configuration can be changed using the "Mouse" menu.  The
  current configuration is described on screen with a small matrix on
  the lower right hand corner, using the following abbreviations:

   Buttons (Horizontal Axis)
   
      L        = left mouse click
      M        = middle mouse click
      R        = right mouse click

   Modifiers (Veritical axis on the matrix) 

      None     = no keys held down while clicking
      Shft     = hold SHIFT down while clicking
      Ctrl     = hold CTRL down while clicking
      CtSh     = hold both SHIFT and CTRL down while clicking
      
   Visualization Functions
   
      Rota     = Rotates camera about X, Y, and Z axes
      RotZ     = Rotates camera about the Z axis
      Move     = Translates along the X and Y axes
      MovZ     = Translates along Z axis
      Clip     = Y motion moves the near clipping plane while
      PkAt     = Pick an atom
      PkBd     = Pick a bond
      Orig     = Move origin to selected atom
      +lb      = Add an atom into the (lb) selection
      lb       = Define the (lb) selection with the indicated atom.
      rb       = Define the (rb) selection with the indicated atom.

   Editing Functions
   
      RotF     = Rotate fragment
      MovF     = Move fragment
      TorF     = Torsion fragment
'''
   cmd.help('mouse')

def examples():
   '''
EXAMPLE ATOM SELECTIONS
 
   select bk = ( name ca or name c or name n )
      * can be abbreviated as *
   sel bk = (n;ca,c,n)
 
   select hev = ( not hydro )
      * can be abbreviated as *
   sel hev = (!h;)
 
   select site = ( byres ( resi 45:52 expand 5 ))
      * can be abbreviated as *
   sel site = (b;(i;45:52 x;5))
 
   select combi = ( hev and not site )
      * can be abbreviated as *
   sel combi = (hev&!site)
   '''
   cmd.help('examples')
   
def launching():
   '''
PyMOL COMMAND LINE OPTIONS 
 
   pymol.com [-ciqstwx] [ <file.pdb> ] [ <file.pml> ] [ <file.py> ] ...
 
   -c   Command line mode, no GUI.  For batch opeations.
   -i   Disable the internal OpenGL GUI (object list, menus, etc.)
   -x   Disable the external GUI module.
   -t   Use Tcl/Tk based external GUI module (pmg_tk).
   -w   Use wxPython based external GUI module (pmg_wx).
   -q   Quiet launch. Suppress splash screen & other chatter.
   -p   Listen for commands on standard input.
   -e   Start in full-screen mode
   -2   Start in two-button mouse mode
   
   -f <# line> Controls display of commands and feedback in OpenGL (0=off).
   -r <file.py> Run a Python program (in __main__) on startup.
   -l <file.py> Spawn a python program in new thread.
   -d <string> Run pymol command string upon startup.
   -u <script> Load and append to this PyMOL script or program file.
   -s <script> Save commands to this PyMOL script or program file.
   
   <file> can have one of the following extensions, and all 
   files provided will be loaded or run after PyMOL starts.
    
    .pml            PyMOL command script to be run on startup
    .py, .pym, .pyc Python program to be run on startup
    .pdb            Protein Data Bank format file to be loaded on startup
    .mmod           Macromodel format to be loaded on startup
    .mol            MDL MOL file to be loaded on startup
    .xplor          X-PLOR Map file (ASCII) to be loaded on startup
    .ccp4           CCP4 map file (BINARY) to be loaded on startup
    .pkl            Pickled ChemPy Model (class "chempy.model.Indexed")
    .r3d            Raster3D Object
    .cc1, .cc2      ChemDraw 3D cartesian coordinate file
   '''
   cmd.help('launching')

def movies():
   '''
MOVIES
 
   To create a movie, simply load multiple coordinate files
   into the same object.  This can be accomplish at the command line,
   using script files, or by writing PyMOL API-based programs.
 
   The commands:
   
load frame001.pdb,mov
load frame002.pdb,mov
 
   will create a two frame movie.  So will the following program:
   
from pymol import cmd
   
for a in ( "frame001.pdb","frame002.pdb" ):
   cmd.load(a,"mov")
 
   which can be executed at the command line using the "run" command.
   
   Python built-in glob module can be useful for loading movies.

from pymol import cmd
import glob
for a in ( glob.glob("frame*.pdb") ):
   cmd.load(a,"mov")
 
NOTE
 
   Because PyMOL stores all movie frames in memory, there is a
   a practical limit to the number of atoms in all coordinate files. 
   160 MB free RAM enables 500,000 atoms with line representations.
   Complex representations require significantly more memory.
   '''
   cmd.help('movies')
   
### -------------------------------------------------------------------
def selections():
   '''
DESCRIPTION
 
   Selections are enclosed in parentheses and contain predicates,
   logical operations, object names, selection names and nested
   parenthesis: ( [... [(...) ... ]] )
 
      name <atom names>            n. <atom names>          
      resn <residue names>         r. <residue names>
      resi <residue identifiers>   i. <residue identifiers>
      chain <chain ID>             c. <chain identifiers>
      segi <segment identifiers>   s. <segment identifiers>
      elem <element symbol>        e. <element symbols>
      flag <number>                f. <number>
      alt <code>                   
      numeric_type <numeric type>  nt. <numeric type>
      text_type <text type>        tt. <text type>
      b <operator> <value>         
      q <operator> <value>         
      formal_charge <op> <value>   fc. <operator> <value>
      partial_charge <op> <value>  pc. <operator> <value>
      id <original-index>          
      hydrogen                     h.
      all                          *
      visible                      v.
      hetatm                       
      <selection> and <selection>  <selection> & <selection>
      <selection> or <selection>   <selection> | <selection>
      not <selection>              ! <selection>
      byres <selection>            br. <selection>
      byobj <selection>            bo. <selection>
      around <distance>            a. <distance>
      expand <distance>            e. <distance>
      gap <distance>               
      in <selection>               
      like <selection>             l. <selection>
      <selection> within <distance> of <selection>
                       <selection> w. <distance> of <selection>      
   '''
   cmd.help('selections')

def povray():
   '''
DESCRIPTION

   PovRay: Persistance of Vision Support Information (EXPERIMENTAL)
   
   The built-in ray-tracer (technically a ray-caster) is as fast or
   faster than PovRay for most figures (provided that hash_max is
   tuned for your system).  However, PovRay blows PyMOL away when it
   comes to rendering images without using lots of RAM, and with
   PovRay you get the ability use perspective, textures, reflections,
   infinite objects, and a superior lighting model.

   PovRay does not include interpolated colors within triangles (!!),
   so you must patch the source code in order to obtain this basic
   functionality required for rendering molecular surfaces.  Details
   can be found on the DINO website.
   
      http://www.biozentrm.unibas.ch/~xray/dino

   To use PovRay, you must be running under Unix and have x-povray in
   your path.  Unfortunately, the developers of PovRay do not share
   PyMOL's open-source philosophy, so you will need to download,
   configure, patch (for smooth_color_triangles), and install it
   yourself.  Despite being free and open-source, PovRay's license
   prevents it from being modified or conveniently combined with
   PyMOL, and thus it serves as a textbook example for why open-source
   licenses should be wholly non-restrictive.

   Assuming that PovRay is built and in your path...

   ray renderer=1   # will use PovRay instead of the built-in engine

   set ray_default_renderer=1 # changes the default renderer to PovRay
   ray                        # will now use PovRay by default

   cmd.get_povray() # will give you a tuple of PovRay input strings
                    # which you can manipulate from Python

   set smooth_color_triangles = 1 # is required in order to get decent
      surfaces in PovRay.  You must patch PovRay first.
      
   '''
   cmd.help('povray')


def faster():
   '''
RAY TRACING OPTIMIZATION

   1. Reduce object complexity to a minimum acceptable level.
         For example, try lowering:
            "cartoon_sampling" 
            "ribbon_sampling", and
            "surface_quality", as appropriate.

   2. Increase "hash_max" so as to obtain a voxel dimensions of
      0.3-0.6.  Proper tuning of "hash_max" can speed up
      rendering by a factor of 2-5X for non-trivial scenes.
      
      WARNING: memory usage depends on hash_max^3, so avoid
      pushing into virtual memory.  Roughly speaking:
      
         hash_max = 80  -->   ~9 MB hash + data
         hash_max = 160 -->  ~72 MB hash + data
         hash_max = 240 --> ~243 MB hash + data

      Avoid utilizing virtual memory for the voxel hash,
      it will slow things way down.
      
   3. Recompiling with optimizations on usually gives a 25-33%
      performance boost for ray tracing.

   '''

   help('faster')



from chempy import Storage,Atom
from chempy.models import Indexed,Connected
import string
import copy

class ARC(Storage):
   
#---------------------------------------------------------------------------------
   def fromFile(self,fname):
      list = []
      f = open(fname)
      while 1:
         hdr = f.readline()
         if not hdr: break
         hdr = hdr[0:6]
         if hdr:
            lst = []
            for b in xrange(int(hdr)):
               a = f.readline()
               lst.append([float(a[11:23]),float(a[23:35]),float(a[35:47])])
            list.append(lst)
      f.close()
      return list
   
   






# 
# basic run through the PDB with wire and cartoon display
#
      
from glob import glob

import threading
import time
import whrandom
from pymol import cmd
import sys, os, os.path

ent_dir = "pdb"

def load():
   list = glob("pdb/*/*")
   l = len(list)
   c = 0
   for file in list:
      c = c + 1
      cmd.delete('pdb')
      cmd.load(file,'pdb')
      cmd.refresh()
      cmd.hide()
      cmd.show('cartoon')
      cmd.color('red','ss h')
      cmd.color('yellow','ss s')
      cmd.orient('pdb')
      sys.__stderr__.write(".")
      sys.__stderr__.flush()
      n = cmd.count_states()
      if n>1:
         for a in range(1,n+1):
            cmd.refresh()
            cmd.frame(a)
            time.sleep(0.025)
         sys.__stderr__.write(" %d of %d"%(c,l))
         sys.__stderr__.write("\n")
         sys.__stderr__.flush()
      else:
         cmd.refresh()
load()





#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import thread
import string

import selector
import pymol

import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from cmd import toggle_dict,toggle_sc

def mstop():
   '''
DESCRIPTION
  
   "mstop" stops the movie.
   
USAGE
 
   mstop
 
PYMOL API
 
   cmd.mstop()

SEE ALSO

   mplay, mset, mdo, mclear, mmatrix
   '''
   try:
      lock()   
      r = _cmd.mplay(0)
   finally:
      unlock()
   return r

def mplay():
   '''
DESCRIPTION
  
   "mplay" starts the movie.
   
USAGE
 
   mplay
 
PYMOL API
 
   cmd.mplay()

SEE ALSO

   mstop, mset, mdo, mclear, mmatrix
   '''
   try:
      lock()   
      r = _cmd.mplay(1)
   finally:
      unlock()
   return r

def mray(): # deprecated
   try:
      lock()   
      r = _cmd.mplay(2)
   finally:
      unlock()
   return r

def mdo(frame,command):
   '''
DESCRIPTION
  
   "mdo" sets up a command to be executed upon entry into the
   specified frame of the movie.  These commands are usually created
   by a PyMOL utility program (such as util.mrock).  Command can
   actually contain several commands separated by semicolons ';'
 
USAGE
 
   mdo frame : command
 
PYMOL API
  
   cmd.mdo( int frame, string command )
 
EXAMPLE
 
   // Creates a single frame movie involving a rotation about X and Y
   
   load test.pdb
   mset 1
   mdo 1, turn x,5; turn y,5;
   mplay
   
NOTES
 
   The "mset" command must first be used to define the movie before
   "mdo" statements will have any effect.  Redefinition of the movie
   clears any existing mdo statements.

SEE ALSO

   mset, mplay, mstop
   '''
   try:
      lock()   
      r = _cmd.mdo(int(frame)-1,str(command),0)
   finally:
      unlock()
   return r

def mappend(frame,command):
   '''
DESCRIPTION
  
USAGE
 
   mappend frame : command
 
PYMOL API
  
EXAMPLE
   
NOTES
 
SEE ALSO

   mset, mplay, mstop
   '''
   try:
      lock()   
      r = _cmd.mdo(int(frame)-1,str(";"+command),1)
   finally:
      unlock()
   return r

def mpng(prefix,first=0,last=0):
   '''
DESCRIPTION
  
   "mpng" writes a series of numbered movie frames to png files with
   the specified prefix.  If the "ray_trace_frames" variable is
   non-zero, these frames will be ray-traced.  This operation can take
   several hours for a long movie.
 
   Be sure to disable "cache_frames" when issuing this operation on a
   long movie (>100 frames) to avoid running out of memory.
   
USAGE
 
   mpng prefix [, first [, last]]

   Options "first" and "last" can be used to specify an inclusive
   interval over which to render frames.  Thus, you can write a smart
   Python program that will automatically distribute rendering over a
   cluster of workstations.  If these options are left at zero, then
   the entire movie will be rendered.

PYMOL API
 
   cmd.mpng( string prefix, int first=0, int last=0 )
   '''
   if thread.get_ident() ==pymol.glutThread:
      r = cmd._mpng(prefix,int(first)-1,int(last)-1)
   else:
      r = _cmd.do("cmd._mpng('"+prefix+","+str(first)+","+str(last)+"')")
   return r

def mclear():
   '''
DESCRIPTION
  
   "mclear" clears the movie frame image cache.
   
USAGE
 
   mclear
 
PYMOL API
 
   cmd.mclear()
   '''
   try:
      lock()   
      r = _cmd.mclear()
   finally:
      unlock()
   return r


def frame(frame):
   '''
DESCRIPTION
  
   "frame" sets the viewer to the indicated movie frame.
   
USAGE
 
   frame frame-number
 
PYMOL API
 
   cmd.frame( int frame_number )
 
NOTES
 
   Frame numbers are 1-based

SEE ALSO

   count_states
   '''
   try:
      lock()   
      r = _cmd.frame(int(frame))
   finally:
      unlock()
   return r

def mset(specification=""):
   '''
DESCRIPTION
  
   "mset" sets up a relationship between molecular states and movie
   frames.  This makes it possible to control which states are shown
   in which frame.
   
USAGE

   mset specification
 
PYMOL API
 
   cmd.mset( string specification )
 
EXAMPLES

   mset 1         // simplest case, one state -> one frame
   mset 1 x10     // ten frames, all corresponding to state 1
   mset 1 x30 1 -15 15 x30 15 -1
     // more realistic example:
     // the first thirty frames are state 1
     // the next 15 frames pass through states 1-15
     // the next 30 frames are of state 15
     // the next 15 frames iterate back to state 1

SEE ALSO

   mdo, mplay, mclear
   '''
   try:
      lock()
      output=[]
      input = string.split(string.strip(specification))
      last = 0
      for x in input:
         if x[0]>"9" or x[0]<"0":
            if x[0]=="x":
               cnt = int(x[1:])-1
               while cnt>0:
                  output.append(str(last))
                  cnt=cnt-1
            elif x[0]=="-":
               dir=1
               cnt=last
               last = int(x[1:])-1
               if last<cnt:
                  dir=-1
               while cnt!=last:
                  cnt=cnt+dir
                  output.append(str(cnt))
         else:
            val = int(x) - 1
            output.append(str(val))
            last=val
      r = _cmd.mset(string.join(output," "))
   finally:
      unlock()
   return r


def mmatrix(action):
   '''
DESCRIPTION
  
   "mmatrix" sets up a matrix to be used for the first frame of the movie.
   
USAGE
 
   mmatrix {clear|store|recall}
 
PYMOL API
 
   cmd.mmatrix( string action )
 
EXAMPLES
 
   mmatrix store
   '''
   r = 1
   try:
      lock()   
      if action=="clear":
         r = _cmd.mmatrix(0)
      elif action=="store":
         r = _cmd.mmatrix(1)
      elif action=="recall":
         r = _cmd.mmatrix(2)
   finally:
      unlock()
   return r


def forward():
   '''
DESCRIPTION
  
   "forward" moves the movie one frame forward.
 
USAGE
 
   forward
 
PYMOL API
  
   cmd.forward()

SEE ALSO

   mset, backward, rewind
   '''
   try:
      lock()   
      r = _cmd.setframe(5,1)
   finally:
      unlock()
   return r

def backward():
   '''
DESCRIPTION
  
   "backward" moves the movie back one frame.
 
USAGE
 
   backward
 
PYMOL API
  
   cmd.backward()

SEE ALSO

   mset, forward, rewind
   '''
   try:
      lock()   
      r = _cmd.setframe(5,-1)
   finally:
      unlock()
   return r


def rewind():
   '''
DESCRIPTION
  
   "rewind" goes to the beginning of the movie.
 
USAGE
 
   rewind
 
PYMOL API
  
   cmd.rewind()
   '''
   try:
      lock()   
      r = _cmd.setframe(4,0)
   finally:
      unlock()
   return r

def ending():
   '''
DESCRIPTION
  
   "ending" goes to the end of the movie.
 
USAGE
 
   ending
 
PYMOL API
  
   cmd.ending()
   '''
   try:
      lock()   
      r=_cmd.setframe(2,0)
   finally:
      unlock()
   return r

def middle():
   '''
DESCRIPTION
  
   "middle" goes to the middle of the movie.
 
USAGE
 
   middle
 
PYMOL API
  
   cmd.middle()
   '''
   try:
      lock()   
      r = _cmd.setframe(3,0)
   finally:
      unlock()
   return r


def get_state():
   '''
DESCRIPTION
  
   "get_state" returns the current state index (1-based)
 
PYMOL API
 
   cmd.get_state()
 
NOTES
 
   States refer to different geometric configurations which an object
   can above.  By default, states and movie frames have a one-to-one
   relationship.  States can be visited in an arbitrary order to
   create frames.  The "mset" command allows you to build a
   relationship between states and frames.

SEE ALSO

   get_frame
   '''
   # NOTE: NO LOCKS...this is/can be called from cmd.refresh()
   r = _cmd.get_state()+1
   return r

def get_frame():
   '''
DESCRIPTION
  
   "get_frame" returns the current frame index (1-based)
 
PYMOL API
 
   Frames refers to sequences of images in a movie.  Sequential frames
   may contain identical molecular states, they may have one-to-one
   correspondance to molecular states (default), or they may have an
   arbitrary relationship, specific using the "mset" command.

SEE ALSO

   get_state
 
   '''
   # NOTE: NO LOCKS...this is/can be be called from cmd.refresh()
   r = _cmd.get_frame()
   return r



from chempy.models import Indexed
from chempy import Storage,Atom,Bond

#
# THIS MODULE IS OBSOLETE PLEASE DO NOT USE
#

import string

atNum = {
   'H'  : 1,
   'C'  : 6,
   'N'  : 7,
   'O'  : 8,
   'F'  : 9,
   'P'  : 15,
   'S'  : 16,
   'Cl' : 17,
   'Br' : 35,
   'I'  : 53,
   }

class GMS(Storage):

   def toList(self,model,runtyp='OPTIMIZE',exetyp='RUN',
              gbasis='N31',ngauss=6,ndfunc=1,dirscf=1):

         
      gmsList = []

      # write header records

      nzvar = (model.nAtom*3)-6
      chg = 0
      for a in model.atom:
         chg = chg + a.formal_charge
      chg = int(chg)
      if chg==0:
         icharg=''
         diffsp=''
      else:
         icharg='ICHARG=%d' % chg
         if chg<0:
            diffsp='DIFFSP=.TRUE.'
         else:
            diffsp=''
      gmsList.append(
         " $CONTRL RUNTYP=%s COORD=UNIQUE EXETYP=%s NZVAR=%d %s $END\n" % 
         (runtyp,exetyp,nzvar,icharg))
      if ndfunc>0:
         gmsList.append(" $BASIS GBASIS=%s NGAUSS=%d NDFUNC=%d %s $END\n" %
                        (gbasis,ngauss,ndfunc,diffsp))
      else:
         gmsList.append(" $BASIS GBASIS=%s NGAUSS=%d %s $END\n",
                        (gbasis,ngauss,diffsp))
      if dirscf:
         gmsList.append(" $SCF DIRSCF=.TRUE. $END\n")
      gmsList.append(" $DATA\n")
      gmsList.append(model.molecule.title+" 6-31G* optimization\n")
      gmsList.append("C1\n")

      # write atom records in an ordering compatible with internal
      # coordinate generation
      c = 1
      for z in model.get_internal_tuples():
         a = model.atom[z[0]]
         if not len(a.name):
            name = a.symbol + "%02d"%c
         else:
            name = a.name
         gmsList.append("%4s %5.2f %12.6f %12.6f %12.6f\n" %
                        (name,atNum[a.symbol],a.coord[0],
                         a.coord[1],a.coord[2]))
         c = c + 1
      gmsList.append(" $END\n")
      gmsList.append(" $ZMAT DLC=.TRUE. AUTO=.TRUE. $END\n")
      if runtyp=='OPTIMIZE':
         gmsList.append(" $STATPT NPRT=-2 NPUN=-2 NSTEP=50 $END\n")
      gmsList.append(" $ELPOT IEPOT=1 WHERE=PDC $END\n")
      gmsList.append(" $PDC PTSEL=GEODESIC CONSTR=CHARGE $END\n")
                     
      return(gmsList)










from pymol import opengl

from _glut import *
try:
    from _glut import *
    
except ImportError:
    print "Importing the GLUT library failed."
    import sys
    if sys.platform == 'win32':
        print "The GLUT32.DLL file should be in your PATH"
    print "Consult the documentation for opengl.glut"
    print "for installation suggestions."
	
from glutconst import *



import sys
from pymol import opengl

"""
This module performs some nasty trickery in order to automatically
detect OpenGL errors, by wrapping each callable function with a
function which calls the function, but afterwards checks to see if an
error code was set, in which case it raises an exception

This is highly experimental.  To disable it, call the fast()
function.  To reenable it, call the careful() function.

"""

if opengl._numeric:
   from Numeric import ArrayType
   try:
      import _opengl_num
      _opengl = _opengl_num
   except ImportError:
      import _opengl
   except SystemError:
      import _opengl
   try:
      import openglutil_num
      openglutil =openglutil_num
   except ImportError:
      import openglutil
   except SystemError:
      import openglutil      
else:
   import _opengl
   import openglutil

origdict = _opengl.__dict__.copy()
origdict.update(openglutil.__dict__)

from glconst import *

_safetylevel = 0

# Constants
import glconst
constant_names = {}
for k, v in glconst.__dict__.items():
   if type(v) == type(1):
      constant_names[v] = k
   
error = origdict['glGetError']
class Error:
   def __init__(self, num):
      self.num = num
      self.str = constant_names.get(num, 'Unknown Error')
   def __str__(self):
      return '%s (%d)' % (self.str, self.num)

class CarefulFunction:
   def __init__(self, name, function):
      self.name = name
      self.func = function
   def __repr__(self):
      return "<careful function wrapper around %s>" % self.function
   def __call__(self, *args, **kw):
      # did someone check in a debug (non-production) version of this module?
      # this was _not_ done in the 1.1 release (causes huge gobs of printing
      # when the demos are run).
##        print "calling %s with %s, %s" % (self.func, args, kw)
      retval = apply(self.func, args, kw)
      err = error()
      if err:
         if _verysafe:
            raise Error(err)
         else:
            print Error(err)
      return retval

carefuldict = {}
for name, func in origdict.items():
   if callable(func):
      carefuldict[name] = CarefulFunction(name, func)
# These do the same sorts of things that the C versions would

def careful():
   cd = carefuldict.copy()
   if cd.has_key('error'): del cd['error']
   sys.modules['pymol.opengl.gl'].__dict__.update(cd)

import string
def fast():
   cd = origdict.copy()
   if cd.has_key('error'): del cd['error']
   if cd.has_key('glconst'): del cd['glconst']
   sys.modules['pymol.opengl.gl'].__dict__.update(cd)

# by default, be careful
if _safetylevel >= 1:
   careful()
else:
   fast()
if _safetylevel >= 2:
   _verysafe = 1
else:
   _verysafe = 0
   
def glGetIntegerv(*args):
   r = apply(glGetDoublev, args)
   if isinstance(r, type(())):
      return tuple(map(int, r))
   elif opengl._numeric and isinstance(r, ArrayType): 
      return tuple(r.astype('i').tolist())
   else:
      return int(r)
glGetInteger = glGetIntegerv
glGetBooleanv = glGetBoolean = glGetFloatv = glGetFloat = glGetDouble

# for b/w compatibility w/ 1.0??
GL_LEVELS = 0x0010

__version__ = '1.5.4'



# pymol

from pymol import cmd

from chempy import io
from chempy import protein,hetatm
from chempy import feedback
from chempy import protein_amber99
from chempy import tinker
from chempy.tinker import keyword
from chempy.tinker.amber import Parameters,Topology,Subset
from chempy.tinker.state import State

import os

state = None
model = None

def assign(sele,preserve=0):

   from molobj import MolObj
   from typer import Typer,Rules
   import rules

   global state
   global model

   result = 1
   
   state = State()

   model = cmd.get_model(sele)
   # now assign atom types

   ruleSet = Rules()

#   ruleSet.fromList(rules.amber_types)
   ruleSet.fromList(rules.simple_types)

   mobj = MolObj()
   mobj.fromChemPyModel(model)

   typed = Typer(molObj = mobj)
   
   print " realtime: assigning atom types"
   typed.applyRules(ruleSet)

   c = 0
   for a in typed.getNamedTypes():
      at = model.atom[c]
      if (at.text_type == '??') or (not preserve):
         if a=='':
            print " warning: unable to assign atom type to atom %d"%c
            result = 0
         else:
            cmd.alter("((%s) and (index %s))" % (sele,at.index),
                      "text_type ='%s'" % a)
            if feedback['tinker']:
               print " "+str(__name__)+': %s is a %s' % (at.name,a)
            at.text_type = a
      c = c + 1

   sm = 0
   for a in model.atom:
      a.resi = str(a.resi_number)
      sm = sm + a.partial_charge

   print " lig: net charge on ligand  is %8.4f\n" % sm

   return result

#   param = Parameters(tinker.params_path+"parm99_wld.dat")
#   param = Parameters(tinker.params_path+"simple_parm.dat")
#   param = Parameters("simple_parm.dat")

def setup(sele,preserve=0):
   
   global state
   global model
   
   state = State()

   model = cmd.get_model(sele)

   sm = 0
   for a in model.atom:
      a.resi = str(a.resi_number)
      sm = sm + a.partial_charge

   print " lig: net charge on ligand  is %8.4f\n" % sm

#   param = Parameters(tinker.params_path+"parm99_wld.dat")
   param = Parameters(tinker.params_path+"parm99_simple.dat")
   topo = Topology(model)

   subset = Subset(param,topo)

   if(subset.complete()):
      subset.write_tinker_prm("realtime.prm")

      state.params = "realtime.prm"

      state.load_model(model)
      return 1
   else:
      subset.dump_missing()
      model = model
      state = None
      return 0

def dyna(steps,iter=1):

   global state

   if not state:
      print " realtime.dyna: please run setup first."
   else:
      state.echo = 0

      model = state.model

      print " realtime.dyna: %d atoms total\n" % model.nAtom

      xtra_kw = []

      xtra_kw.extend(keyword.get_inactive(model,3))
      xtra_kw.extend(keyword.get_restrain_positions(model,2,0,10))
      xtra_kw.extend(keyword.get_restrain_positions(model,5,0.5,1))
      xtra_kw.extend(keyword.get_inactive(model,6))

      state.keywords['chg-cutoff'] = 10.0
      state.keywords['vdw-cutoff'] = 7.00
      state.keywords['lights'] = ''
      state.keywords['restrainterm'] = ''

      for x in range(0,iter):
         state.dynamics(steps=steps,timestep=1,kw=xtra_kw)
         if not len(state.summary):
            break
         for a in state.summary:
            print a
         cmd.load_model(model,'dyna')
         cmd.ending()
         cmd.refresh()
      io.pkl.toFile("realtime.pkl")
      print " realtime.dyna: terminated after %d steps." % state.counter

def check(obj='check'):
   global state
   global model
   
   if not state:
      if not model:
         print " realtime.reload: please run setup first."
      else:
         cmd.load_model(model,obj,1)
   else:
      model = state.model
      cmd.load_model(model,obj,1)

   
def mini(total_step=100,gradient=0.001,interval=100,obj='rt'):

   global state

   if not state:
      print " realtime.mini: please run setup first."
   else:
      state.echo = 0

      model = state.model

      print " realtime.mini: %d atoms total\n" % model.nAtom

      xtra_kw = []

      xtra_kw.extend(keyword.get_inactive(model,3))
      xtra_kw.extend(keyword.get_restrain_positions(model,2,0,10))
      xtra_kw.extend(keyword.get_restrain_positions(model,5,0.5,1))
      xtra_kw.extend(keyword.get_inactive(model,6))

      state.keywords['chg-cutoff'] = 10.0
      state.keywords['vdw-cutoff'] = 7.00
      state.keywords['lights'] = ''
      state.keywords['restrainterm'] = ''

      iter = total_step/interval
      for x in range(0,iter):
         state.minimize(gradient=gradient,max_iter=interval,kw=xtra_kw)
         cmd.delete(obj)
         cmd.load_model(model,obj,1)
         cmd.refresh()
         if not len(state.summary):
            break
         for a in state.summary:
            print a
         if state.summary[-1][7]=='SmallGrad':
            break;         
      io.pkl.toFile(model,"realtime.pkl")
      print " realtime.mini: terminated after %d steps." % state.counter







#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import string
import selector
import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException

button_code = {
   'left' : 0,
   'middle' : 1,
   'right' : 2,
   }
button_sc = Shortcut(button_code.keys())

but_mod_code = {
   'none'  : 0,
   'shft'  : 1,
   'ctrl'  : 2,
   'ctsh'  : 3
   }
but_mod_sc = Shortcut(but_mod_code.keys())

but_act_code = {
   'rota' :  0 ,
   'move' :  1 ,
   'movz' :  2 ,
   'clip' :  3 ,
   'rotz' :  4 ,
   'clpn' :  5 ,
   'clpf' :  6 ,
   'lb'   :  7 ,
   'mb'   :  8 ,
   'rb'   :  9 ,
   '+lb'  : 10 ,
   '+mb'  : 11 ,
   '+rb'  : 12 ,
   'pkat' : 13 ,
   'pkbd' : 14 ,
   'rotf' : 15 ,
   'torf' : 16 ,
   'movf' : 17 ,
   'orig' : 18 ,
   '+lbx' : 19 ,
   '-lbx' : 20 ,
   'lbbx' : 21 ,
   'none' : 22 ,
   }
but_act_sc = Shortcut(but_act_code.keys())


ring_dict = {
   'three_button' : [   'three_button_viewing',
                        'three_button_editing' ],
   'two_button' : [ 'two_button_viewing',
                    'two_button_selecting',
                            ],
   'two_button_editing' : [ 'two_button_viewing',
                            'two_button_selecting',
                            'two_button_editing',
                            ]
   }

def config_mouse(mode='three_button',quiet=1):
   global mouse_ring
   if ring_dict.has_key(mode):
      mouse_ring = ring_dict[mode]
      if not quiet:
         print " config_mouse: %s"%mode
      mouse(quiet=1)
         
   else:
      print " Error: unrecognized mouse ring: '%s'"%mode

mouse_ring = ring_dict['three_button']

mode_dict = {
   'three_button_viewing' : [ ('l','none','rota'),
                              ('m','none','move'),
                              ('r','none','movz'),
                              ('l','shft','+lbx'),
                              ('m','shft','-lbx'),
                              ('r','shft','clip'),                 
                              ('l','ctrl','+lb'),
                              ('m','ctrl','pkat'),
                              ('r','ctrl','pkbd'),                 
                              ('l','ctsh','lb'),
                              ('m','ctsh','orig'),
                              ('r','ctsh','rb')
                              ],
   
   'three_button_editing': [ ('l','none','rota'),
                             ('m','none','move'),
                             ('r','none','movz'),
                             ('l','shft','rotf'),
                             ('m','shft','movf'),
                             ('r','shft','clip') ,                 
                             ('l','ctrl','torf'),
                             ('m','ctrl','pkat'),
                             ('r','ctrl','pkbd'),                  
                             ('l','ctsh','lb'),
                             ('m','ctsh','orig'),
                             ('r','ctsh','rb'),
                             ],
   
   'two_button_viewing' : [ ('l','none','rota'),
                            ('m','none','none'),
                            ('r','none','movz'),
                            ('l','shft','pkat'),
                            ('m','shft','none'),
                            ('r','shft','clip'),                 
                            ('l','ctrl','move'),
                            ('m','ctrl','none'),
                            ('r','ctrl','pkbd'),                 
                            ('l','ctsh','lb'),
                            ('m','ctsh','none'),
                            ('r','ctsh','orig')
                            ],
   'two_button_selecting' : [ ('l','none','rota'),
                              ('m','none','none'),
                              ('r','none','movz'),
                              ('l','shft','+lbx'),
                              ('m','shft','none'),
                              ('r','shft','-lbx'),                 
                              ('l','ctrl','+lb'),
                              ('m','ctrl','none'),
                              ('r','ctrl','+rb'),                 
                              ('l','ctsh','lb'),
                              ('m','ctsh','none'),
                              ('r','ctsh','rb')
                            ],
   'two_button_editing' : [ ('l','none','rota'),
                            ('m','none','none'),
                            ('r','none','movz'),
                            ('l','shft','pkat'),
                            ('m','shft','none'),
                            ('r','shft','clip'),                 
                            ('l','ctrl','torf'),
                            ('m','ctrl','none'),
                            ('r','ctrl','pkbd'),                 
                            ('l','ctsh','rotf'),
                            ('m','ctsh','none'),
                            ('r','ctsh','movf')
                            ],
   }

def mouse(action=None,quiet=1):# INTERNAL
   # NOTE: PyMOL automatically runs this routine upon start-up
   try:
      lock()
      
      if action=='forward':
         bm = _cmd.get_setting("button_mode")
         bm = (int(bm) + 1) % len(mouse_ring)
         cmd.set("button_mode",str(bm),quiet=1)
         action=None
      elif action=='backward':
         bm = _cmd.get_setting("button_mode")
         bm = (int(bm) - 1) % len(mouse_ring)
         cmd.set("button_mode",str(bm),quiet=1)
         action=None

      mode_list = None
      if action==None:
         bm = _cmd.get_setting("button_mode")
         bm = int(bm) % len(mouse_ring)
         mode = mouse_ring[bm]
         mode_list = mode_dict[mode]
      elif action in mode_dict.keys():
         mode = action
         mode_list = mode_dict[mode]
      if mode_list!=None:
         for a in mode_list:
            apply(button,a)
         if not quiet:
            print " mouse: %s"%mode
   finally:
      unlock()

def edit_mode(mode=None):
   # legacy function
   if len(mouse_ring):
      mouse_mode = mouse_ring[0]
      if mouse_mode[0:10]=='two_button':
         mouse(action='two_button_editing',quiet=0)
      elif mouse_mode[0:12] == 'three_button':
         mouse(action='three_button_editing',quiet=0)

#   try:
#      lock()
#      r = _cmd.get_setting("button_mode")
#      r = int(r)
#      if mode==None:
#         if r:
#            _cmd.legacy_set("button_mode","0")
#         else:
#            _cmd.legacy_set("button_mode","1")            
#      else:
#         if mode=='on':
#            _cmd.legacy_set("button_mode","1")
#         if mode=='off':
#            _cmd.legacy_set("button_mode","0")
#      config_mouse()
#   finally:
#      unlock()
#   pass

def set_key(key,fn,arg=(),kw={}):  
   '''
DESCRIPTION
  
   "set_key" binds a specific python function to a key press.
   
PYMOL API (ONLY)
 
   cmd.set_key( string key, function fn, tuple arg=(), dict kw={})
 
PYTHON EXAMPLE
 
   from pymol import cmd
 
   def color_blue(object): cmd.color("blue",object)
    
   cmd.set_key( 'F1' , make_it_blue, ( "object1" ) )
   // would turn object1 blue when the F1 key is pressed and

   cmd.set_key( 'F2' , make_it_blue, ( "object2" ) )
   // would turn object2 blue when the F2 key is pressed.

   cmd.set_key( 'CTRL-C' , cmd.zoom )   
   cmd.set_key( 'ALT-A' , cmd.turn, ('x',90) )

KEYS WHICH CAN BE REDEFINED

   F1 to F12
   left, right, pgup, pgdn, home, insert
   CTRL-A to CTRL-Z 
   ALT-0 to ALT-9, ALT-A to ALT-Z
   
SEE ALSO

   button
   '''
   r = 0
   if key[0:5]=='CTRL-': 
      pat=key[-1]
      for a in cmd.ctrl.keys():
         if a==pat:
            cmd.ctrl[a][0]=fn
            cmd.ctrl[a][1]=arg
            cmd.ctrl[a][2]=kw
            r = 1
   elif key[0:4]=='ALT-':
      pat=string.lower(key[-1])
      for a in cmd.alt.keys():
         if a==pat:
            cmd.alt[a][0]=fn
            cmd.alt[a][1]=arg
            cmd.alt[a][2]=kw
            r = 1
   else:
      if key[0]!='F':
         pat=string.lower(key)
      else:
         pat=key
      for a in cmd.special.keys():
         if cmd.special[a][0]==pat:
            cmd.special[a][1]=fn
            cmd.special[a][2]=arg
            cmd.special[a][3]=kw
            r = 1
            
   if not r:
      print "Error: special '%s' key not found."%key
      if cmd._raising(): raise QuietException
   return r
   
def button(button,modifier,action):
   '''
DESCRIPTION
  
   "button" can be used to redefine what the mouse buttons do.
   
USAGE
 
   button <button>,<modifier>,<action>
 
PYMOL API
 
   cmd.button( string button, string modifier, string action )
 
NOTES

   button:      L, M, R
   modifers:    None, Shft, Ctrl, CtSh
   actions:     Rota, Move, MovZ, Clip, RotZ, ClpN, ClpF
                lb,   mb,   rb,   +lb,  +lbX, -lbX, +mb,  +rb, 
                PkAt, PkBd, RotF, TorF, MovF, Orig

'''
   r=1
   try:
      lock()
      button = string.lower(button)
      button = button_sc.auto_err(button,'button')
      modifier = string.lower(modifier)
      modifier = but_mod_sc.auto_err(modifier,'modifier')
      action = string.lower(action)
      action = but_act_sc.auto_err(action,'action')
      but_code = button_code[button] + 3*but_mod_code[modifier]
      act_code = but_act_code[action]
      r = _cmd.button(but_code,act_code)
   finally:
      unlock()
   return r
   

def mask(selection="(all)"):
   '''
DESCRIPTION
  
   "mask" makes it impossible to select the indicated atoms using the
   mouse.  This is useful when you are working with one molecule in
   front of another and wish to avoid accidentally selecting atoms in
   the background.

USAGE

   mask (selection)

PYMOL API

   cmd.mask( string selection="(all)" )
   
SEE ALSO

   unmask, protect, deprotect, mouse
'''
   # preprocess selection
   selection = selector.process(selection)
   #
   try:
      lock()   
      r = _cmd.mask("("+str(selection)+")",1)
   finally:
      unlock()
   return r

def unmask(selection="(all)"):
   '''
DESCRIPTION

   "unmask" reverses the effect of "mask" on the indicated atoms.

PYMOL API

   cmd.unmask( string selection="(all)" )
   
USAGE

   unmask (selection)

SEE ALSO

   mask, protect, deprotect, mouse
'''
   # preprocess selection
   selection = selector.process(selection)
   #   
   try:
      lock()   
      r = _cmd.mask("("+str(selection)+")",0)
   finally:
      unlock()
   return r




#A*
#-------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program C*
#copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific.  D*
#-------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.  F*
#-------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information.
#H*
#-------------------------------------------------------------------
#I* Additional authors of this source file include: -* -* NOTE: Based
#on code by John E. Grayson which was in turn -* based on code written
#by Doug Hellmann.  Z*
#-------------------------------------------------------------------

# this section is devoted to making sure that Tkinter variables which
# correspond to Menu-displayed settings are kept synchronized with
# PyMOL

from Tkinter import *
import Pmw
from pymol import cmd
import pymol.setting

pm = cmd

class Setting:

   def __init__(self):
   
      self.ray_trace_frames = IntVar()
      self.ray_trace_frames.set(int(cmd.get_setting_legacy('ray_trace_frames')))
      
      self.cache_frames = IntVar()
      self.cache_frames.set(int(cmd.get_setting_legacy('ray_trace_frames')))

      self.ortho = IntVar()
      self.ortho.set(int(cmd.get_setting_legacy('orthoscopic')))

      self.antialias = IntVar()
      self.antialias.set(int(cmd.get_setting_legacy('antialias')))

      self.all_states = IntVar()
      self.all_states.set(int(cmd.get_setting_legacy('all_states')))

      self.line_smooth = IntVar()
      self.line_smooth.set(int(cmd.get_setting_legacy('line_smooth')))

      self.overlay = IntVar()
      self.overlay.set(int(cmd.get_setting_legacy('overlay')))

      self.valence = IntVar()
      self.valence.set(int(cmd.get_setting_legacy('valence')))

      self.auto_zoom = IntVar()
      self.auto_zoom.set(int(cmd.get_setting_legacy('auto_zoom')))

      self.auto_show_selections = IntVar()
      self.auto_show_selections.set(int(cmd.get_setting_legacy('auto_show_selections')))

      self.auto_hide_selections = IntVar()
      self.auto_hide_selections.set(int(cmd.get_setting_legacy('auto_hide_selections')))

      self.static_singletons = IntVar()
      self.static_singletons.set(int(cmd.get_setting_legacy('static_singletons')))

      self.backface_cull = IntVar()
      self.backface_cull.set(int(cmd.get_setting_legacy('backface_cull')))

      self.depth_cue = IntVar()
      self.depth_cue.set(int(cmd.get_setting_legacy('depth_cue')))

      self.specular = IntVar()
      self.specular.set(int(cmd.get_setting_legacy('specular')))

      self.cartoon_round_helices = IntVar()
      self.cartoon_round_helices.set(int(cmd.get_setting_legacy('cartoon_round_helices')))

      self.cartoon_fancy_helices = IntVar()
      self.cartoon_fancy_helices.set(int(cmd.get_setting_legacy('cartoon_fancy_helices')))

      self.cartoon_flat_sheets = IntVar()
      self.cartoon_flat_sheets.set(int(cmd.get_setting_legacy('cartoon_flat_sheets')))

      self.cartoon_fancy_sheets = IntVar()
      self.cartoon_fancy_sheets.set(int(cmd.get_setting_legacy('cartoon_fancy_sheets')))

      self.cartoon_discrete_colors = IntVar()
      self.cartoon_discrete_colors.set(int(cmd.get_setting_legacy('cartoon_discrete_colors')))

      self.cartoon_smooth_loops = IntVar()
      self.cartoon_smooth_loops.set(int(cmd.get_setting_legacy('cartoon_smooth_loops')))

      self.cartoon_cylindrical_helices = IntVar()
      self.cartoon_cylindrical_helices.set(int(cmd.get_setting_legacy('cartoon_cylindrical_helices')))

      self.ignore_pdb_segi = IntVar()
      self.ignore_pdb_segi.set(int(cmd.get_setting_legacy('ignore_pdb_segi')))

      self.log_box_selections = IntVar()
      self.log_box_selections.set(int(cmd.get_setting_legacy('log_box_selections')))

      self.log_conformations = IntVar()
      self.log_conformations.set(int(cmd.get_setting_legacy('log_conformations')))

      self.two_sided_lighting = IntVar()
      self.two_sided_lighting.set(int(cmd.get_setting_legacy('two_sided_lighting')))

      self.auto_remove_hydrogens = IntVar()
      self.auto_remove_hydrogens.set(int(cmd.get_setting_legacy('auto_remove_hydrogens')))

      self.auto_sculpt = IntVar()
      self.auto_sculpt.set(int(cmd.get_setting_legacy('auto_sculpt')))

      self.sculpting = IntVar()
      self.sculpting.set(int(cmd.get_setting_legacy('sculpting')))

      self.xref = { 
         'ray_trace_frames':
         (lambda s,a: (cmd.set(a,("%1.0f" % s.ray_trace_frames.get()),log=1),
                       s.cache_frames.set(s.ray_trace_frames.get()),
                       s.update('cache_frames'))),
         'cache_frames'  :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.cache_frames.get()),log=1))),
         'ortho'         :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.ortho.get()),log=1))),
         'antialias'     :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.antialias.get()),log=1))),
         'valence'       :
         (lambda s,a: (cmd.set(a,("%1.2f" % (float(s.valence.get())/20.0)),log=1))),
         'all_states'    :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.all_states.get()),log=1))),
         'line_smooth'    :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.line_smooth.get()),log=1))),
         'overlay'       :
         (lambda s,a: (cmd.set(a,("%1.0f" %( s.overlay.get()*5)),log=1))),
         'auto_zoom'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.auto_zoom.get()),log=1))),
         'auto_show_selections'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.auto_show_selections.get()),log=1))),
         'auto_hide_selections'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.auto_hide_selections.get()),log=1))),
         'static_singletons'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.static_singletons.get()),log=1))),
         'backface_cull'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % s.backface_cull.get()),log=1))),
         'depth_cue'       :
         (lambda s,a: s.depth_cue_set(a)),
         'specular'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.specular.get()*0.8)),log=1))),

         'cartoon_round_helices'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_round_helices.get())),log=1))),
         'cartoon_fancy_helices'       :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_fancy_helices.get())),log=1))),
         'cartoon_flat_sheets'         :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_flat_sheets.get())),log=1))),
         'cartoon_fancy_sheets'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_fancy_sheets.get())),log=1))),
         'cartoon_discrete_colors'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_discrete_colors.get())),log=1))),
         'cartoon_smooth_loops'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_smooth_loops.get())),log=1))),
         'cartoon_cylindrical_helices'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.cartoon_cylindrical_helices.get())),log=1))),

         'ignore_pdb_segi'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.ignore_pdb_segi.get())),log=1))),
         'log_box_selections'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.log_box_selections.get())),log=1))),
         'log_conformations'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.log_conformations.get())),log=1))),

         'two_sided_lighting'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.two_sided_lighting.get())),log=1))),

         'auto_remove_hydrogens'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.auto_remove_hydrogens.get())),log=1))),
         'auto_sculpt'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.auto_sculpt.get())),log=1))),
         'sculpting'        :
         (lambda s,a: (cmd.set(a,("%1.0f" % (s.sculpting.get())),log=1))),
         }

      self.update_code = {
         'ray_trace_frames':
         (lambda s,t: (s.ray_trace_frames.set(int(t[1][0])))),
         'cache_frames':
         (lambda s,t: (s.cache_frames.set(int(t[1][0])))),
         'orthoscopic':
         (lambda s,t: (s.ortho.set(int(t[1][0])))),
         'all_states':
         (lambda s,t: (s.all_states.set(int(t[1][0])))),
         'line_smooth':
         (lambda s,t: (s.line_smooth.set(int(t[1][0])))),
         'overlay':
         (lambda s,t: (s.overlay.set(t[1][0]!=0))),
         'valence':
         (lambda s,t: (s.valence.set(t[1][0]>0.0))),
         'auto_zoom':
         (lambda s,t: (s.auto_zoom.set(int(t[1][0])))), 
         'auto_show_selections':
         (lambda s,t: (s.auto_show_selections.set(int(t[1][0])))), 
         'auto_hide_selections':
         (lambda s,t: (s.auto_hide_selections.set(int(t[1][0])))), 
         'static_singletons':
         (lambda s,t: (s.static_singletons.set(int(t[1][0])))), 
         'backface_cull':
         (lambda s,t: (s.backface_cull.set(int(t[1][0])))), 
         'depth_cue'       :
         (lambda s,t: (s.depth_cue.set(int(t[1][0])))),
         'specular'       :
         (lambda s,t: (s.specular.set(t[1][0]>0.0))), 
         'cartoon_round_helices':
         (lambda s,t: (s.cartoon_round_helices.set(t[1][0]!=0))),
         'cartoon_fancy_helices':
         (lambda s,t: (s.cartoon_fancy_helices.set(t[1][0]!=0))),
         'cartoon_flat_sheets':
         (lambda s,t: (s.cartoon_flat_sheets.set(t[1][0]!=0))),
         'cartoon_fancy_sheets':
         (lambda s,t: (s.cartoon_fancy_sheets.set(t[1][0]!=0))),
         'cartoon_discrete_colors':
         (lambda s,t: (s.cartoon_discrete_colors.set(t[1][0]!=0))),
         'cartoon_smooth_loops':
         (lambda s,t: (s.cartoon_smooth_loops.set(t[1][0]!=0))),
         'cartoon_cylindrical_helices':
         (lambda s,t: (s.cartoon_cylindrical_helices.set(t[1][0]!=0))),
         'ignore_pdb_segi':
         (lambda s,t: (s.ignore_pdb_segi.set(t[1][0]!=0))),
         'log_box_selections':
         (lambda s,t: (s.log_box_selections.set(t[1][0]!=0))),
         'log_conformations':
         (lambda s,t: (s.log_conformations.set(t[1][0]!=0))),
         'two_sided_lighting':
         (lambda s,t: (s.two_sided_lighting.set(t[1][0]!=0))),
         'auto_remove_hydrogens':
         (lambda s,t: (s.auto_remove_hydrogens.set(t[1][0]!=0))),
         'auto_sculpt':
         (lambda s,t: (s.auto_sculpt.set(t[1][0]!=0))),
         'sculpting':
         (lambda s,t: (s.sculpting.set(t[1][0]!=0))),

        }
      self.active_list = [
         pymol.setting._get_index("ray_trace_frames"),
         pymol.setting._get_index("cache_frames"),
         pymol.setting._get_index("orthoscopic"),
         pymol.setting._get_index("antialias"),
         pymol.setting._get_index("all_states"),
         pymol.setting._get_index("overlay"),
         pymol.setting._get_index("line_smooth"),
         pymol.setting._get_index("valence"),
         pymol.setting._get_index("auto_zoom"),
         pymol.setting._get_index("auto_hide_selections"),
         pymol.setting._get_index("auto_show_selections"),
         pymol.setting._get_index("static_singletons"),
         pymol.setting._get_index("backface_cull"),
         pymol.setting._get_index("depth_cue"),
         pymol.setting._get_index("specular"),
         pymol.setting._get_index("cartoon_round_helices"),
         pymol.setting._get_index("cartoon_fancy_helices"),
         pymol.setting._get_index("cartoon_flat_sheets"),
         pymol.setting._get_index("cartoon_fancy_sheets"),
         pymol.setting._get_index("cartoon_discrete_colors"),
         pymol.setting._get_index("cartoon_smooth_loops"),
         pymol.setting._get_index("cartoon_cylindrical_helices"),                  
         pymol.setting._get_index("ignore_pdb_segi"),         
         pymol.setting._get_index("log_box_selections"),
         pymol.setting._get_index("log_conformations"),
         pymol.setting._get_index("two_sided_lighting"),
         pymol.setting._get_index("auto_remove_hydrogens"),
         pymol.setting._get_index("auto_sculpt"),
         pymol.setting._get_index("sculpting"),                  
         ]

      self.active_dict = {}
      for a in self.active_list:
         self.active_dict[a] = pymol.setting._get_name(a)

   def depth_cue_set(self,sttng):
      cmd.set(sttng,("%1.0f" % self.depth_cue.get()),log=1)
      cmd.set("ray_trace_fog",("%1.0f" % self.depth_cue.get()),log=1)
                         
   def update(self,sttng):
      set_fn = self.xref[sttng]
      set_fn(self,sttng)

   def refresh(self): # get any settings changes from PyMOL and update menus
      lst = cmd.get_setting_updates()
      for a in lst:
         if a in self.active_list:
            name = self.active_dict[a]
            if self.update_code.has_key(name):
               code = self.update_code[name]
               if code!=None:
                  tup = cmd.get_setting_tuple(name,'',0)
                  if tup!=None:
                     apply(code,(self,tup))
      



# Generated by h2py from glut.h
GLUT_API_VERSION = 3
GLUT_XLIB_IMPLEMENTATION = 12
GLUT_RGB = 0
GLUT_RGBA = GLUT_RGB
GLUT_INDEX = 1
GLUT_SINGLE = 0
GLUT_DOUBLE = 2
GLUT_ACCUM = 4
GLUT_ALPHA = 8
GLUT_DEPTH = 16
GLUT_STENCIL = 32
GLUT_MULTISAMPLE = 128
GLUT_STEREO = 256
GLUT_LUMINANCE = 512
GLUT_LEFT_BUTTON = 0
GLUT_MIDDLE_BUTTON = 1
GLUT_RIGHT_BUTTON = 2
GLUT_DOWN = 0
GLUT_UP = 1
GLUT_KEY_F1 = 1
GLUT_KEY_F2 = 2
GLUT_KEY_F3 = 3
GLUT_KEY_F4 = 4
GLUT_KEY_F5 = 5
GLUT_KEY_F6 = 6
GLUT_KEY_F7 = 7
GLUT_KEY_F8 = 8
GLUT_KEY_F9 = 9
GLUT_KEY_F10 = 10
GLUT_KEY_F11 = 11
GLUT_KEY_F12 = 12
GLUT_KEY_LEFT = 100
GLUT_KEY_UP = 101
GLUT_KEY_RIGHT = 102
GLUT_KEY_DOWN = 103
GLUT_KEY_PAGE_UP = 104
GLUT_KEY_PAGE_DOWN = 105
GLUT_KEY_HOME = 106
GLUT_KEY_END = 107
GLUT_KEY_INSERT = 108
GLUT_LEFT = 0
GLUT_ENTERED = 1
GLUT_MENU_NOT_IN_USE = 0
GLUT_MENU_IN_USE = 1
GLUT_NOT_VISIBLE = 0
GLUT_VISIBLE = 1
GLUT_HIDDEN = 0
GLUT_FULLY_RETAINED = 1
GLUT_PARTIALLY_RETAINED = 2
GLUT_FULLY_COVERED = 3
GLUT_RED = 0
GLUT_GREEN = 1
GLUT_BLUE = 2
GLUT_NORMAL = 0
GLUT_OVERLAY = 1
GLUT_WINDOW_X = 100
GLUT_WINDOW_Y = 101
GLUT_WINDOW_WIDTH = 102
GLUT_WINDOW_HEIGHT = 103
GLUT_WINDOW_BUFFER_SIZE = 104
GLUT_WINDOW_STENCIL_SIZE = 105
GLUT_WINDOW_DEPTH_SIZE = 106
GLUT_WINDOW_RED_SIZE = 107
GLUT_WINDOW_GREEN_SIZE = 108
GLUT_WINDOW_BLUE_SIZE = 109
GLUT_WINDOW_ALPHA_SIZE = 110
GLUT_WINDOW_ACCUM_RED_SIZE = 111
GLUT_WINDOW_ACCUM_GREEN_SIZE = 112
GLUT_WINDOW_ACCUM_BLUE_SIZE = 113
GLUT_WINDOW_ACCUM_ALPHA_SIZE = 114
GLUT_WINDOW_DOUBLEBUFFER = 115
GLUT_WINDOW_RGBA = 116
GLUT_WINDOW_PARENT = 117
GLUT_WINDOW_NUM_CHILDREN = 118
GLUT_WINDOW_COLORMAP_SIZE = 119
GLUT_WINDOW_NUM_SAMPLES = 120
GLUT_WINDOW_STEREO = 121
GLUT_WINDOW_CURSOR = 122
GLUT_SCREEN_WIDTH = 200
GLUT_SCREEN_HEIGHT = 201
GLUT_SCREEN_WIDTH_MM = 202
GLUT_SCREEN_HEIGHT_MM = 203
GLUT_MENU_NUM_ITEMS = 300
GLUT_DISPLAY_MODE_POSSIBLE = 400
GLUT_INIT_WINDOW_X = 500
GLUT_INIT_WINDOW_Y = 501
GLUT_INIT_WINDOW_WIDTH = 502
GLUT_INIT_WINDOW_HEIGHT = 503
GLUT_INIT_DISPLAY_MODE = 504
GLUT_ELAPSED_TIME = 700
GLUT_HAS_KEYBOARD = 600
GLUT_HAS_MOUSE = 601
GLUT_HAS_SPACEBALL = 602
GLUT_HAS_DIAL_AND_BUTTON_BOX = 603
GLUT_HAS_TABLET = 604
GLUT_NUM_MOUSE_BUTTONS = 605
GLUT_NUM_SPACEBALL_BUTTONS = 606
GLUT_NUM_BUTTON_BOX_BUTTONS = 607
GLUT_NUM_DIALS = 608
GLUT_NUM_TABLET_BUTTONS = 609
GLUT_OVERLAY_POSSIBLE = 800
GLUT_LAYER_IN_USE = 801
GLUT_HAS_OVERLAY = 802
GLUT_TRANSPARENT_INDEX = 803
GLUT_NORMAL_DAMAGED = 804
GLUT_OVERLAY_DAMAGED = 805
GLUT_VIDEO_RESIZE_POSSIBLE = 900
GLUT_VIDEO_RESIZE_IN_USE = 901
GLUT_VIDEO_RESIZE_X_DELTA = 902
GLUT_VIDEO_RESIZE_Y_DELTA = 903
GLUT_VIDEO_RESIZE_WIDTH_DELTA = 904
GLUT_VIDEO_RESIZE_HEIGHT_DELTA = 905
GLUT_VIDEO_RESIZE_X = 906
GLUT_VIDEO_RESIZE_Y = 907
GLUT_VIDEO_RESIZE_WIDTH = 908
GLUT_VIDEO_RESIZE_HEIGHT = 909
GLUT_ACTIVE_SHIFT = 1
GLUT_ACTIVE_CTRL = 2
GLUT_ACTIVE_ALT = 4
GLUT_CURSOR_RIGHT_ARROW = 0
GLUT_CURSOR_LEFT_ARROW = 1
GLUT_CURSOR_INFO = 2
GLUT_CURSOR_DESTROY = 3
GLUT_CURSOR_HELP = 4
GLUT_CURSOR_CYCLE = 5
GLUT_CURSOR_SPRAY = 6
GLUT_CURSOR_WAIT = 7
GLUT_CURSOR_TEXT = 8
GLUT_CURSOR_CROSSHAIR = 9
GLUT_CURSOR_UP_DOWN = 10
GLUT_CURSOR_LEFT_RIGHT = 11
GLUT_CURSOR_TOP_SIDE = 12
GLUT_CURSOR_BOTTOM_SIDE = 13
GLUT_CURSOR_LEFT_SIDE = 14
GLUT_CURSOR_RIGHT_SIDE = 15
GLUT_CURSOR_TOP_LEFT_CORNER = 16
GLUT_CURSOR_TOP_RIGHT_CORNER = 17
GLUT_CURSOR_BOTTOM_RIGHT_CORNER = 18
GLUT_CURSOR_BOTTOM_LEFT_CORNER = 19
GLUT_CURSOR_INHERIT = 100
GLUT_CURSOR_NONE = 101
GLUT_CURSOR_FULL_CROSSHAIR = 102




import os
import shutil
import glob
import re
import string
import sys
import time
import socket  # for gethostname()
import getpass # for getuser()
import re

from chempy import feedback

# "do" is the preferred command for running tinker

def do(run_prefix):
   if feedback['bmin']:
      print " "+str(__name__)+': launching %s...'%bmin_path
      if hasattr(sys.stdout,"flush"):
         sys.stdout.flush()
   for a in [ ".m1", ".m2", ".log", ".out" ]:
      if os.path.exists(run_prefix + a):
         os.unlink(run_prefix + a)
   pth = os.getcwd()
   pth = re.sub(r".*\/"+getpass.getuser()+"\/",'',pth)
   cmd = "rsh "+socket.gethostname()+" "+bmin_path+" "+pth+"/"+run_prefix
   print cmd
   os.system(cmd)
   if feedback['bmin']:
      os.system("cat bmintmp.log")
      print " "+str(__name__)+': bmin job complete. ' 
      if hasattr(sys.stdout,"flush"):
         sys.stdout.flush()

if os.environ.has_key('SCHRODINGER'):
   base = os.environ['SCHRODINGER']
   bmin_path = base + '/bmin'
#   os.environ['MMSHARE_EXEC'] = '/apps/schrodinger/mmshare-v10028/bin/Linux-x86'
#   os.environ['MMOD_EXEC'] = '/apps/schrodinger/macromodel-v71008/bin/Linux-x86'
#   bmin_path = os.environ['MMOD_EXEC']+"/bmin"
else:
   bmin_path = "bmin"




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import selector

import operator
import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from cmd import file_ext_re
from chempy import fragments

map_type_dict = {
   'vdw' : 0,
   'coulomb' : 1,
   }

map_type_sc = Shortcut(map_type_dict.keys())

def map_new(name,type,grid=None,selection=None,buffer=0.0,box=None):
   # preprocess selection
   r = None
   if selection!=None:
      selection = selector.process(selection)
   else:
      selection = ""
   if box!=None: # box should be [[x1,y1,z1],[x2,y2,z2]]
      if cmd.is_string(box):
         box = eval(box)
      box = (float(box[0][0]),
             float(box[0][1]),
             float(box[0][2]),
             float(box[1][0]),
             float(box[1][1]),
             float(box[1][2]))
   else:
      box = (0.0,0.0,0.0,1.0,1.0,1.0)
   grid = float(grid) # for now, uniform xyz; later (x,y,z)
   type = map_type_dict[map_type_sc.auto_err(str(type),'map type')]
   try:
      lock()
      r = _cmd.map_new(str(name),int(type),grid,str(selection),float(buffer),box)
   finally:
      unlock()
      
def isomesh(name,map,level=1.0,selection='',buffer=0.0,state=0,carve=None):
   '''
DESCRIPTION
 
   "isomesh" creates a mesh isosurface object from a map object.
 
USAGE
 
   isomesh name, map, level [,(selection) [,buffer [,state [,carve ]]]]

   "name" is the name for the new mesh isosurface object.
   
   "map" is the name of the map object to use for computing the mesh.
   
   "level" is the contour level.

   "selection" is an atom selection about which to display the mesh with
      an additional "buffer" (if provided).

   "state" is the state into which the object should be loaded.

   "carve" is a radius about each atom in the selection for which to
      include density. If "carve" is not provided, then the whole
      brick is displayed.

NOTES

   If the mesh object already exists, then the new mesh will be
   appended onto the object as a new state (unless you indicate a state).

SEE ALSO

   isodot, load
   '''
   if selection!='':
      mopt = 1 # about a selection
   else:
      mopt = 0 # render the whole map
   # preprocess selection
   selection = selector.process(selection)
   #
   if carve==None:
      carve=-1.0
   try:
      lock()
      r = _cmd.isomesh(str(name),0,str(map),int(mopt),
                       "("+str(selection)+")",float(buffer),
                       float(level),0,int(state)-1,float(carve))
   finally:
      unlock()
   return r

def isosurface(name,map,level=1.0,selection='',buffer=0.0,state=0,carve=None):
   '''
DESCRIPTION
 
   "isosurface" creates a new surface object from a map object.
 
USAGE
 
   isosurface name, map, level [,(selection) [,buffer [,state [,carve ]]]]

   "name" is the name for the new mesh isosurface object.
   
   "map" is the name of the map object to use for computing the mesh.
   
   "level" is the contour level.

   "selection" is an atom selection about which to display the mesh with
      an additional "buffer" (if provided).

   "state" is the state into which the object should be loaded.

   "carve" is a radius about each atom in the selection for which to
      include density. If "carve" is not provided, then the whole
      brick is displayed.

NOTES

   If the surface object already exists, then the new surface will be
   appended onto the object as a new state (unless you indicate a state).

SEE ALSO

   isodot, isomesh, load
   '''
   if selection!='':
      mopt = 1 # about a selection
   else:
      mopt = 0 # render the whole map
   # preprocess selection
   selection = selector.process(selection)
   #
   if carve==None:
      carve=-1.0
   try:
      lock()
      r = _cmd.isosurface(str(name),0,str(map),int(mopt),
                       "("+str(selection)+")",float(buffer),
                       float(level),0,int(state)-1,float(carve))
   finally:
      unlock()
   return r

def isodot(name,map,level=1.0,selection='',buffer=0.0,state=0,carve=0):
   '''
DESCRIPTION
 
"isodot" creates a dot isosurface object from a map object.
 
USAGE
 
   isodot name = map, level [,(selection) [,buffer [, state ] ] ] 

   "map" is the name of the map object to use.
   
   "level" is the contour level.
   
   "selection" is an atom selection about which to display the mesh with
      an additional "buffer" (if provided).

NOTES

   If the dot isosurface object already exists, then the new dots will
   be appended onto the object as a new state.

SEE ALSO

   load, isomesh
   '''
   if selection!='':
      mopt = 1 # about a selection
   else:
      mopt = 0 # render the whole map
   # preprocess selections
   selection = selector.process(selection)
   #
   try:
      lock()
      r = _cmd.isomesh(str(name),0,str(map),int(mopt),
                       "("+str(selection)+")",float(buffer),
                       float(level),1,int(state)-1,float(carve))
   finally:
      unlock()
   return r

def copy(target,source):
   '''
DESCRIPTION
 
   "copy" creates a new object that is an identical copy of an
   existing object
 
USAGE

   copy target, source

   copy target = source         # (DEPRECATED)
 
PYMOL API
 
   cmd.copy(string target,string source)

SEE ALSO

   create
   '''
   try:
      lock()
      r = _cmd.copy(str(source),str(target))
   finally:
      unlock()
   return r

def symexp(prefix,object,selection,cutoff):
   '''
DESCRIPTION
 
   "symexp" creates all symmetry related objects for the specified object
   that occurs within a cutoff about an atom selection.  The new objects
   are labeled using the prefix provided along with their crystallographic
   symmetry operation and translation.
 
USAGE
 
   symexp prefix = object, (selection), cutoff
 
PYMOL API
 
   cmd.symexp( string prefix, string object, string selection, float cutoff) 

SEE ALSO

   load
   '''
   # preprocess selection
   selection=selector.process(selection)
   #
   try:
      lock()
      r = _cmd.symexp(str(prefix),str(object),"("+str(selection)+")",float(cutoff))
   finally:
      unlock()
   return r

def fragment(name,object=None,origin=1,zoom=0):
   '''
DESCRIPTION
 
   "fragment" retrieves a 3D structure from the fragment library, which is currently
   pretty meager (just amino acids).
   
USAGE
 
   fragment name

'''
   r = 1
   try:
      save=cmd.get_setting_legacy('auto_zoom')
      cmd.set('auto_zoom',zoom,quiet=1)
      try:
         if object==None:
            object=name
         model = fragments.get(str(name))
         la = len(model.atom)
         if la:
            mean = map(lambda x,la=la:x/la,[
               reduce(operator.__add__,map(lambda a:a.coord[0],model.atom)),

               reduce(operator.__add__,map(lambda a:a.coord[1],model.atom)),
               reduce(operator.__add__,map(lambda a:a.coord[2],model.atom))])
            position = cmd.get_position()
            for c in range(0,3):
               mean[c]=position[c]-mean[c]
               map(lambda a,x=mean[c],c=c:cmd._adjust_coord(a,c,x),model.atom)
            mean = map(lambda x,la=la:x/la,[
               reduce(operator.__add__,map(lambda a:a.coord[0],model.atom)),
               reduce(operator.__add__,map(lambda a:a.coord[1],model.atom)),
               reduce(operator.__add__,map(lambda a:a.coord[2],model.atom))])
         cmd.load_model(model,str(object))
      finally:
         cmd.set('auto_zoom',save,quiet=1)
   except:
      print "Error: unable to load fragment %s" % name
      r = 0
   if not r:
      if cmd._raising(): raise QuietException
   return r

def create(name,selection,source_state=0,target_state=0):
   '''
DESCRIPTION
  
   "create" creates a new molecule object from a selection.  It can
   also be used to create states in an existing object.
 
   NOTE: this command has not yet been throughly tested.
 
USAGE

   create name, (selection) [,source_state [,target_state ] ]
   
   create name = (selection) [,source_state [,target_state ] ]
     # (DEPRECATED)

   name = object to create (or modify)
   selection = atoms to include in the new object
   source_state (default: 0 - copy all states)
   target_state (default: 0)
   
PYMOL API
  
   cmd.create(string name, string selection, int state, int target_state)

NOTES

   If the source and target states are zero (default), all states will
   be copied.  Otherwise, only the indicated states will be copied.

SEE ALSO

   load, copy
   '''
   # preprocess selection
   selection = selector.process(selection)
   #      
   try:
      lock()
      if name==None:
         sel_cnt = _cmd.get("sel_counter") + 1.0
         _cmd.legacy_set("sel_counter","%1.0f" % sel_cnt)
         name = "obj%02.0f" % sel_cnt
      _cmd.create(str(name),"("+str(selection)+")",
                  int(source_state)-1,int(target_state)-1)
   finally:
      unlock()
   return None







#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import imp
import sys
import string
import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException,_raising, \
     _feedback,fb_module,fb_mask

def wizard(name):
   '''
DESCRIPTION
 
   "wizard" launches on of the built-in wizards.  There are special
   Python scripts which work with PyMOL in order to obtain direct user
   interaction and easily peform complicated tasks.
   
USAGE
 
   wizard name

PYMOL API

   cmd.wizard(string name)

EXAMPLE

   wizard distance  # launches the distance measurement wizard
'''
   import wizard
   try:
      if not sys.modules.has_key(name):
         mod_tup = imp.find_module(name,wizard.__path__)
         mod_obj = imp.load_module(name,mod_tup[0],mod_tup[1],mod_tup[2])
      else:
         mod_obj = sys.modules[name]
      if mod_obj:
         oname = string.capitalize(name)
         if hasattr(mod_obj,oname):
            wiz = apply(getattr(mod_obj,oname))
            if wiz:
               set_wizard(wiz)
               cmd.do("refresh")
      else:
         print "Error: Sorry, couldn't import the '"+name+"' wizard."         
   except ImportError:
      print "Error: Sorry, couldn't import the '"+name+"' wizard."


def set_wizard(*arg): # INTERNAL
   r = None
   wiz = None
   if len(arg):
      wiz=arg[0]
   try:
      lock()
      r = _cmd.set_wizard(wiz)
   finally:
      unlock()
   return r

def refresh_wizard(*arg): # INTERNAL
   r = None
   wiz = None
   if len(arg):
      wiz=arg[0]
   try:
      lock()
      r = _cmd.refresh_wizard()
   finally:
      unlock()
   return r

def get_wizard(*arg): # INTERNAL
   r = None
   try:
      lock()
      r = _cmd.get_wizard()
   finally:
      unlock()
   return r






#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import selector
from cmd import _cmd,lock,unlock,Shortcut,QuietException
import cmd
import threading

def get_bond_print(obj,max_bond,max_type):
   try:
      lock()
      r = _cmd.get_bond_print(str(obj),int(max_bond),int(max_type))
   finally:
      unlock()
   return r

def expfit(a,b): # Huh?
   try:
      lock()   
      r = _cmd.fit(a,b,2)
   finally:
      unlock()
   return r

def focus():  # BROKEN
   try:
      lock()
      r = _cmd.focus()
   finally:
      unlock()
   return r

def spheroid(object=""):  # EXPERIMENTAL
   try:
      print "Warning: 'spheroid' is experimental, incomplete, and unstable."
      lock()
      r = _cmd.spheroid(str(object))
   finally:
      unlock()
   return r

def mem():
   '''
DESCRIPTION

   "mem" Dumps current memory state to standard output. This is a
   debugging feature, not an official part of the API.

'''
   try:
      lock()
      r = _cmd.mem()
   finally:
      unlock()
   return r

   
def check(selection=None,preserve=0):
# UNSUPPORTED
# This function relies on code that is not currently part of PyMOL/ChemPy
   # NOTE: the realtime module relies on code that is not yet part of PyMOL/ChemPy
   from chempy.tinker import realtime
   if selection==None:
      arg = cmd.get_names("objects")
      arg = arg[0:1]
      if arg:
         if len(arg):
            selection = arg
   if selection!=None:
      selection = selector.process(selection)
      realtime.assign("("+selection+")",int(preserve))
      realtime.setup("("+selection+")")

def fast_minimize(*arg):
# OBSOLETE, TO BE REMOVED
   from chempy.tinker import realtime  
   grad  = 0.01
   iter = 500
   interval = 50
   la = len(arg)
   if not la:
      arg = cmd.get_names("objects")
      arg = arg[0:1]
      la = len(arg)
   if la:
      sele  = "("+arg[0]+")"
      if la>1:
         iter = int(arg[1])
      if la>2:
         grad = float(arg[2])
      if la>3:
         interval = int(arg[3])
      t = threading.Thread(target=realtime.mini,args=(iter,grad,interval,arg[0]))
      t.setDaemon(1)
      t.start()
   
def minimize(*arg):
# OBSOLETE, TO BE REMOVED
   # NOTE: the realtime module relies on code that is not yet part of PyMOL/ChemPy
   from chempy.tinker import realtime  
   grad  = 0.01
   iter = 500
   interval = 50
   la = len(arg)
   if not la:
      arg = cmd.get_names("objects")
      arg = arg[0:1]
      la = len(arg)
   if la:
      sele  = "("+arg[0]+")"
      if la>1:
         iter = int(arg[1])
      if la>2:
         grad = float(arg[2])
      if la>3:
         interval = int(arg[3])
      if realtime.setup(sele):
         t = threading.Thread(target=realtime.mini,args=(iter,grad,interval,arg[0]))
         t.setDaemon(1)
         t.start()
      else:
         print " minimize: missing parameters, can't continue"


def dump(fnam,obj):
   try:
      lock()
      r = _cmd.dump(str(fnam),obj)
   finally:
      unlock()
   return r


def dummy(*arg):
   return None

def test(group=0,index=0): # generic test routine for development
   try:
      lock()   
      r=_cmd.test(int(group),int(index))
   finally:
      unlock()
   return r

def import_coords(obj,state,mechio): # experimental
   r = None
   try:
      lock()   
      r = _cmd.import_coords(str(obj),int(state)-1,mechio)
   finally:
      unlock()
   return r

def load_coords(*arg): # UNSUPPORTED
   r = 1
   try:
      lock()
      ok = 1
      ftype = loadable.model
      state = 0
      model = arg[0];
      if len(arg)<2:
         ok=0
      if len(arg)>=2:
         oname = string.strip(arg[1])
      if len(arg)>=3:
         state = int(arg[2])-1
      if ok:
         r = _cmd.load_coords(str(oname),model,
                              int(state)-1,int(ftype))
      else:
         print "Error: invalid arguments."
   finally:
      unlock()
   return r



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import selector

import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from cmd import _feedback,fb_module,fb_mask

def get_title(object,state,quiet=1):
   '''
DESCRIPTION

   "get_title" retrieves a text string to the state of a particular
   object which will be displayed when the state is active.

USAGE
   
   set_title object,state

PYMOL API

   cmd.set_title(string object,int state,string text)

'''
   r = None
   try:
      lock()
      r = _cmd.get_title(str(object),int(state)-1)
      if not quiet:
         if r!=None:
            print " get_title: %s"%r      
   finally:
      unlock()
   return r

def transform_object(name,matrix,state=0,log=0,sele=''):
   r = None
   try:
      lock()
      r = _cmd.transform_object(str(name),int(state)-1,list(matrix),int(log),str(sele))
   finally:
      unlock()
   return r

def translate_atom(sele1,v0,v1,v2,state=0,mode=0,log=0):
   r = None
   sele1 = selector.process(sele1)
   try:
      lock()
      r = _cmd.translate_atom(str(sele1),float(v0),float(v1),float(v2),int(state)-1,int(mode),int(log))
   finally:
      unlock()
   return r

def distance(name=None,selection1="(lb)",selection2="(rb)",cutoff=None,
             mode=None,zoom=0,width=None,length=None,gap=None):
   '''
DESCRIPTION
 
   "distance" creates a new distance object between two
   selections.  It will display all distances within the cutoff.
 
USAGE
 
   distance 
   distance (selection1), (selection2)
   distance name = (selection1), (selection1) [,cutoff [,mode] ]
 
   name = name of distance object 
   selection1,selection2 = atom selections
   cutoff = maximum distance to display
   mode = 0 (default)

PYMOL API
 
   cmd.distance( string name, string selection1, string selection2,
          string cutoff, string mode )
   returns the average distance between all atoms/frames
 
NOTES

   The distance wizard makes measuring distances easier than using
   the "dist" command for real-time operations.

   "dist" alone will show distances between selections (lb) and (rb)
   created by left and right button atom picks.  CTRL-SHIFT/left-click
   on the first atom,  CTRL-SHIFT/right-click on the second, then run
   "dist".

'''
   # handle unnamed distance 
   r = 1
   if name!=None:
      if len(name):
         if name[0]=='(' or ' ' in name or '/' in name: # we're one argument off...
            if cutoff!=None:
               mode = cutoff
            if selection2!="(rb)":
               cutoff = selection2
            if selection1!="(lb)":
               selection2 = selection1
            selection1=name
            name = None

   if selection1=="(lb)":
      if "lb" not in cmd.get_names('selections'):
         if _feedback(fb_module.cmd,fb_mask.errors):
            print "cmd-Error: The 'lb' selection is undefined."
         r = 0
   if selection2=="(rb)":
      if "rb" not in cmd.get_names('selections'):
         if _feedback(fb_module.cmd,fb_mask.errors):         
            print "cmd-Error: The 'rb' selection is undefined."
         r = 0
   if r:         
      save=cmd.get_setting_legacy('auto_zoom')
      cmd.set('auto_zoom',zoom,quiet=1)

      # if unlabeled, then get next name in series

      if name!=None:
         nam=name
      else:
         try:
            lock()
            cnt = _cmd.get("dist_counter") + 1.0
            _cmd.legacy_set("dist_counter","%1.0f" % cnt)
            nam = "dist%02.0f" % cnt
         finally:
            unlock()

      # defaults
      if mode == None:
         mode = 0
      if cutoff == None:
         cutoff = -1.0
      # preprocess selections
      selection1 = selector.process(selection1)
      selection2 = selector.process(selection2)
      # now do the deed
      try:
         lock()
         r = _cmd.dist(str(nam),"("+str(selection1)+")",
                       "("+str(selection2)+")",int(mode),float(cutoff))
         if width!=None:
            cmd.set("dash_width",width,nam)
         if length!=None:
            cmd.set("dash_length",length,nam)
         if gap!=None:
            cmd.set("dash_gap",gap,nam)
      finally:
         unlock()
      cmd.set('auto_zoom',save,quiet=1)
   if r<0.0:
      if cmd._raising(): raise QuietException
   return r

# LEGACY support for cmd.dist
def dist(*arg,**kw):
   return apply(distance,arg,kw)

def get_povray():
   '''
DESCRIPTION
  
   "get_povray" returns a tuple corresponding to strings for a PovRay
   input file.
      
PYMOL API
  
   cmd.get_povray()

   '''
   r=None
   try:
      lock()   
      r = _cmd.get_povray()
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   return r

def count_states(selection="(all)",quiet=1):
   '''
DESCRIPTION

   "count_states" is an API-only function which returns the number of
   states in the selection.

PYMOL API

   cmd.count_states(string selection="(all)")

SEE ALSO

   frame
'''
   # preprocess selection
   selection = selector.process(selection)
   #
   r = -1
   try:
      lock()
      r = _cmd.count_states(selection)
   finally:
      unlock()
   if r<0:
      if cmd._raising(): raise QuietException
   elif not quiet: print " cmd.count_states: %d states."%r            
   return r

def count_frames(quiet=1):
   '''
DESCRIPTION

   "count_frames" is an API-only function which returns the number of
   frames defined for the PyMOL movie.

PYMOL API

   cmd.count_frames()

SEE ALSO

   frame, count_states
'''
   r = -1
   try:
      lock()
      r = _cmd.count_frames()
      if not quiet: print " cmd.count_frames: %d frames"%r      
   finally:
      unlock()
   if r<0:
      if cmd._raising(): raise QuietException
   return r

def export_dots(object,state):  
# UNSUPPORTED
   try:
      lock()
      r = _cmd.export_dots(object,int(state)-1)
   finally:
      unlock()
   return r

def overlap(selection1,selection2,state1=1,state2=1,adjust=0.0,quiet=1):
#
#   UNSUPPORTED FEATURE - LIKELY TO CHANGE
#   (for maximum efficiency, use smaller molecule as selection 1)
#
   # preprocess selections
   selection1 = selector.process(selection1)
   selection2 = selector.process(selection2)
   #
   r = 1
   try:
      lock()
      r = _cmd.overlap(str(selection1),str(selection2),
                       int(state1)-1,int(state2)-1,
                       float(adjust))
      if not quiet: print " cmd.overlap: %5.3f Angstroms."%r
   finally:
      unlock()
   return r

def get_color_tuple(name):
   r = None
   try:
      lock()
      r = _cmd.get_color(name,0)
      if r==None:
         if _feedback(fb_module.cmd,fb_mask.errors):         
            print "cmd-Error: Unknown color '%s'."%name
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   return r

def get_color_indices():
   r = None
   try:
      lock()
      r = _cmd.get_color('',1)
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   return r

def get_renderer():  # 
   try:
      lock()
      r = _cmd.get_renderer()
   finally:
      unlock()
   return r

def get_phipsi(selection="(name ca)",state=-1):
   # preprocess selections
   selection = selector.process(selection)
   #   
   r = None
   try:
      lock()
      r = _cmd.get_phipsi("("+str(selection)+")",int(state)-1)
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   return r

def get_position(quiet=1):
   r = None
   try:
      lock()
      r = _cmd.get_position()
   finally:
      unlock()
   if r==None:
      if cmd._raising(): raise QuietException
   elif not quiet:
      print " cmd.get_position: [%8.3f,%8.3f,%8.3f]"%(r[0],r[1],r[2])
   return r

def get_dihedral(atom1,atom2,atom3,atom4,state=1,quiet=1):
   # preprocess selections
   atom1 = selector.process(atom1)
   atom2 = selector.process(atom2)
   atom3 = selector.process(atom3)
   atom4 = selector.process(atom4)
   #   
   r = None
   try:
      lock()
      r = _cmd.get_dihe(str(atom1),str(atom2),str(atom3),str(atom4),int(state)-1)
   finally:
      unlock()
   if r==None:
      if cmd._raising(): raise QuietException
   elif not quiet:
      print " cmd.get_dihedral: %5.3f degrees."%r
   return r

def get_model(selection="(all)",state=1):
   '''
DESCRIPTION
  
   "get_model" returns a ChemPy "Indexed" format model from a selection.
 
PYMOL API
 
   cmd.get_model(string selection [,int state] )
 
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = None
   try:
      lock()
      r = _cmd.get_model("("+str(selection)+")",int(state)-1)
   finally:
      unlock()
   if r==None:
      if cmd._raising(): raise QuietException
   return r

def get_area(selection="(all)",state=1,load_b=0,quiet=1):
   '''
   PRE-RELEASE functionality - API will change
   '''
   # preprocess selection
   selection = selector.process(selection)
   #
   r = -1.0
   try:
      lock()
      r = _cmd.get_area("("+str(selection)+")",int(state)-1,int(load_b))
   finally:
      unlock()
   if r<0.0:
      if cmd._raising(): raise QuietException
   elif not quiet: print " cmd.get_area: %5.3f Angstroms^2."%r
   return r

def get_names(type='objects'):
   '''
DESCRIPTION
  
   "get_names" returns a list of object and/or selection names.
 
PYMOL API
 
   cmd.get_names( [string: "objects"|"selections"|"all"] )
 
NOTES
 
   The default behavior is to return only object names.

SEE ALSO

   get_type, count_atoms, count_states
   '''
   mode = 1
   if type=='objects':
      mode = 1
   elif type=='selections':
      mode = 2
   elif type=='all':
      mode = 0
   elif type=='public':
      mode = 3
   try:
      lock()
      r = _cmd.get_names(int(mode))
   finally:
      unlock()
   return r

def get_type(name,quiet=1):
   '''
DESCRIPTION
  
   "get_type" returns a string describing the named object or
    selection or the string "nonexistent" if the name in unknown.
 
PYMOL API
 
   cmd.get_type(string object-name)
 
NOTES

   Possible return values are
   
   "object:molecule"
   "object:map"
   "object:mesh"
   "object:distance"
   "selection"

SEE ALSO

   get_names
   '''
   r = None
   try:
      lock()
      r = _cmd.get_type(str(name))
   finally:
      unlock()
   if not r:
      if _feedback(fb_module.cmd,fb_mask.errors):      
         print "cmd-Error: unrecognized name."
      if cmd._raising(): raise QuietException
   elif not quiet:
      print r
   return r

def id_atom(selection,mode=0,quiet=1):
   '''
DESCRIPTION
  
   "id_atom" returns the original source id of a single atom, or
   raises and exception if the atom does not exist or if the selection
   corresponds to multiple atoms.
 
PYMOL API
 
   list = cmd.id_atom(string selection)
   '''
   r = -1
   selection = str(selection)
   l = apply(identify,(selection,mode,1))
   ll = len(l)
   if not ll:
      if _feedback(fb_module.cmd,fb_mask.errors):
         print "cmd-Error: atom %s not found by id_atom." % selection
      if cmd._raising(): raise QuietException
   elif ll>1:
      if _feedback(fb_module.cmd,fb_mask.errors):
         print "cmd-Error: multiple atoms %s found by id_atom." % selection
      if cmd._raising(): raise QuietException
   else:
      r = l[0]
      if not quiet:
         if mode:
            print " cmd.id_atom: (%s and id %d)"%(r[0],r[1])
         else:
            print " cmd.id_atom: (id %d)"%r
   return r

def identify(selection="(all)",mode=0,quiet=1):
   '''
DESCRIPTION
  
   "identify" returns a list of atom IDs corresponding to the ID code
   of atoms in the selection.
 
PYMOL API
 
   list = cmd.identify(string selection="(all)",int mode=0)

NOTES

   mode 0: only return a list of identifiers (default)
   mode 1: return a list of tuples of the object name and the identifier

   '''
   # preprocess selection
   selection = selector.process(selection)
   #      
   r = []
   try:
      lock()
      r = _cmd.identify("("+str(selection)+")",int(mode)) # 0 = default mode
   finally:
      unlock()
   if len(r):
      if not quiet:
         if mode:
            for a in r:
               print " cmd.identify: (%s and id %d)"%(a[0],a[1])
         else:
            for a in r:
               print " cmd.identify: (id %d)"%a
   return r

def index(selection="(all)",quiet=1):
   '''
DESCRIPTION
  
   "index" returns a list of tuples corresponding to the
   object name and index of the atoms in the selection.
 
PYMOL API
 
   list = cmd.index(string selection="(all)")

NOTE

  Atom indices are fragile and will change as atoms are added
  or deleted.  Whenever possible, use integral atom identifiers
  instead of indices.
  
   '''
   # preprocess selection
   selection = selector.process(selection)
   #      
   r = []
   try:
      lock()
      r = _cmd.index("("+str(selection)+")",0) # 0 = default mode
   finally:
      unlock()
   if not quiet:
      if r:
         for a in r:
            print " cmd.index: (%s`%d)"%(a[0],a[1])
   return r

def find_pairs(selection1,selection2,state1=1,state2=1,cutoff=3.5,mode=0,angle=45):
   '''
DESCRIPTION
  
   "find_pairs" is currently undocumented.
 
   '''
   # preprocess selection
   selection1 = selector.process(selection1)
   selection2 = selector.process(selection2)
   #      
   r = []
   try:
      lock()
      r = _cmd.find_pairs("("+str(selection1)+")",
                          "("+str(selection2)+")",
                          int(state1)-1,int(state2)-1,
                          int(mode),float(cutoff),float(angle))
      # 0 = default mode
   finally:
      unlock()
   return r

def get_extent(selection="(all)",state=0,quiet=1):
   '''
DESCRIPTION
  
   "get_extent" returns the minimum and maximum XYZ coordinates of a
   selection as an array:
    [ [ min-X , min-Y , min-Z ],[ max-X, max-Y , max-Z ]]
 
PYMOL API
 
   cmd.get_extent(string selection="(all)", state=0 )
 
   '''
   # preprocess selection
   selection = selector.process(selection)
   #      
   r = None
   try:
      lock()
      r = _cmd.get_min_max("("+str(selection)+")",int(state)-1)
   finally:
      unlock()
   if not r:
      if cmd._raising(): raise QuietException
   elif not quiet:
      print " cmd.extent: min: [%8.3f,%8.3f,%8.3f]"%(r[0][0],r[0][1],r[0][2])
      print " cmd.extent: max: [%8.3f,%8.3f,%8.3f]"%(r[1][0],r[1][1],r[1][2])      
   return r

def phi_psi(selection="(byres pk1)"):
   result = cmd.get_phipsi(selection)
   if result!=None:
      kees = result.keys()
      kees.sort()
      cmd.feedback('push')
      cmd.feedback('disable','executive','actions')
      for a in kees:
         cmd.iterate("(%s`%d)"%a,"print ' %-9s "+("( %6.1f, %6.1f )"%result[a])+"'%(resn+'-'+resi+':')")
      cmd.feedback('pop')
   elif _feedback(fb_module.cmd,fb_mask.errors):      
      print "cmd-Error: can't compute phi_psi"
   return result


def count_atoms(selection="(all)",quiet=1):
   '''
DESCRIPTION
  
   "count_atoms" returns a count of atoms in a selection.
 
USAGE
 
   count_atoms (selection)
 
PYMOL API
  
   cmd.count(string selection)
 
   '''
   # preprocess selection
   selection = selector.process(selection)
   #
   try:
      lock()   
      r = _cmd.select("_count_tmp","("+str(selection)+")",1)
      _cmd.delete("_count_tmp")
   finally:
      unlock()
   if not quiet: print " count_atoms: %d atoms"%r
   return r

def get_names_of_type(type):
   obj = cmd.get_names('objects')
   types = map(get_type,obj)
   mix = map(None,obj,types)
   lst = []
   for a in mix:
      if a[1]==type:
         lst.append(a[0])
   return lst

